import json
import os
import shutil
import textwrap

import pytest

from conan.api.subapi.workspace import WorkspaceAPI
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.mocks import ConanFileMock
from conan.test.utils.scm import create_local_git_repo
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save_files
from conan.tools.files import replace_in_file

WorkspaceAPI.TEST_ENABLED = "will_break_next"


class TestWorkspaceRoot:

    def test_workspace_root(self):
        c = TestClient(light=True)
        # Just check the root command works
        c.run("workspace root", assert_error=True)
        assert "ERROR: Workspace not defined, please create" in c.out

        # error, conanws/ folder does not contain conanws.[py | yml]
        c.save({"conanws/test.txt": ""})
        with c.chdir("conanws"):
            c.run("workspace root", assert_error=True)
            assert "ERROR: Workspace not defined, please create" in c.out
            c.save({"conanws.yml": ""})
            c.run("workspace root")
            assert c.current_folder in c.stdout

        # error, empty .py
        c.save({"conanws.py": ""}, clean_first=True)
        c.run("workspace root", assert_error=True)
        assert "Error loading conanws.py" in c.out
        assert "No subclass of Workspace" in c.out

        c.save({"conanws.yml": ""}, clean_first=True)
        c.run("workspace root")
        assert c.current_folder in c.stdout


class TestAddRemove:

    def test_add(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "dep1/conanfile.py": GenConanfile("dep1", "0.1"),
                "dep2/conanfile.py": GenConanfile("dep2", "0.1"),
                "dep3/conanfile.py": GenConanfile("dep3", "0.1")})
        c.run("workspace add dep1")
        assert "Reference 'dep1/0.1' added to workspace" in c.out
        c.run("workspace info")
        assert "dep1/0.1" in c.out
        assert "dep2" not in c.out
        c.run("editable list")  # No editables in global
        assert "dep1" not in c.out
        assert "dep2" not in c.out
        c.run("workspace add dep2")
        assert "Reference 'dep2/0.1' added to workspace" in c.out
        c.run("workspace info")
        assert "dep1/0.1" in c.out
        assert "dep2/0.1" in c.out

        with c.chdir(temp_folder()):  # If we move to another folder, outside WS, no editables
            c.run("editable list")
            assert "dep1" not in c.out
            assert "dep2" not in c.out

        c.run("workspace info")
        assert "dep1/0.1" in c.out
        assert "dep2/0.1" in c.out

        c.run("workspace remove dep1")
        c.run("workspace info")
        assert "dep1/0.1" not in c.out
        assert "dep2/0.1" in c.out

        c.run("workspace remove dep2")
        c.run("workspace info")
        assert "dep1/0.1" not in c.out
        assert "dep2/0.1" not in c.out

    def test_add_from_outside(self):
        c = TestClient(light=True)
        c.save({"sub/conanws.yml": "",
                "sub/dep1/conanfile.py": GenConanfile("dep1", "0.1"),
                "sub/dep2/conanfile.py": GenConanfile("dep2", "0.1")})
        with c.chdir("sub"):
            c.run("workspace add dep1")
            assert "Reference 'dep1/0.1' added to workspace" in c.out
            c.run("workspace add dep2")
            assert "Reference 'dep2/0.1' added to workspace" in c.out
            c.run("workspace info")
            assert "dep1/0.1" in c.out
            assert "dep2/0.1" in c.out
        assert c.load_home("editable_packages.json") is None

        c.run("editable list")
        assert "dep1" not in c.out
        assert "dep2" not in c.out
        assert c.load_home("editable_packages.json") is None
        with c.chdir("sub"):
            c.run("editable add dep1")
            assert c.load_home("editable_packages.json") is not None
            c.run("editable list")
            assert "dep1/0.1" in c.out
            assert "dep2/0.1" not in c.out
            c.run("workspace info")
            assert "dep1" in c.out
            assert "dep2" in c.out

        c.run("editable list")
        assert "dep1/0.1" in c.out
        assert "dep2" not in c.out

    def test_add_modified(self):
        # https://github.com/conan-io/conan/issues/18942
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "dep1/conanfile.py": GenConanfile("dep1", "0.1")})
        c.run("workspace add dep1")
        assert "Reference 'dep1/0.1' added to workspace" in c.out
        c.run("workspace info")
        assert "dep1/0.1" in c.out
        c.save({"dep1/conanfile.py": GenConanfile("dep1", "0.2")})
        c.run("workspace add dep1")
        assert "Package dep1 already exists, updating its reference" in c.out
        assert "Reference 'dep1/0.2' added to workspace" in c.out
        c.run("workspace info")
        assert "dep1/0.2" in c.out
        assert "dep1/0.1" not in c.out

    @pytest.mark.parametrize("api", [False, True])
    def test_dynamic_editables(self, api):
        c = TestClient(light=True)
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import load
            class Lib(ConanFile):
                def set_name(self):
                    self.name = load(self, os.path.join(self.recipe_folder, "name.txt"))
                def set_version(self):
                    self.version = load(self, os.path.join(self.recipe_folder, "version.txt"))
            """)
        if not api:
            workspace = textwrap.dedent("""\
                import os
                from conan import Workspace

                class MyWorkspace(Workspace):
                    def packages(self):
                        result = []
                        for f in os.listdir(self.folder):
                            if os.path.isdir(os.path.join(self.folder, f)):
                                full_path = os.path.join(self.folder, f, "name.txt")
                                name = open(full_path).read().strip()
                                version = open(os.path.join(self.folder, f,
                                                            "version.txt")).read().strip()
                                result.append({"path": f, "ref": f"{name}/{version}"})
                        return result
                """)
        else:
            workspace = textwrap.dedent("""\
                import os
                from conan import Workspace

                class MyWorkspace(Workspace):
                    def packages(self):
                        result = []
                        for f in os.listdir(self.folder):
                            if os.path.isdir(os.path.join(self.folder, f)):
                                conanfile = self.load_conanfile(f)
                                result.append({"path": f,
                                               "ref": f"{conanfile.name}/{conanfile.version}"})
                        return result
               """)

        c.save({"conanws.py": workspace,
                "dep1/conanfile.py": conanfile,
                "dep1/name.txt": "pkg",
                "dep1/version.txt": "2.1"})
        c.run("workspace info --format=json")
        info = json.loads(c.stdout)
        assert info["packages"] == [{"ref": "pkg/2.1", "path": "dep1"}]
        c.save({"dep1/name.txt": "other",
                "dep1/version.txt": "14.5"})
        c.run("workspace info --format=json")
        info = json.loads(c.stdout)
        assert info["packages"] == [{"ref": "other/14.5", "path": "dep1"}]
        c.run("install --requires=other/14.5")
        # Doesn't fail
        assert "other/14.5 - Editable" in c.out
        with c.chdir("dep1"):
            c.run("install --requires=other/14.5")
            # Doesn't fail
            assert "other/14.5 - Editable" in c.out

    def test_api_dynamic_version_run(self):
        # https://github.com/conan-io/conan/issues/17306
        c = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from io import StringIO
            from conan import ConanFile
            class Lib(ConanFile):
                name= "pkg"
                def set_version(self):
                    my_buf = StringIO()
                    self.run('echo 2.1', stdout=my_buf)
                    self.version = my_buf.getvalue().strip()
            """)

        workspace = textwrap.dedent("""\
            import os
            from conan import Workspace

            class MyWorkspace(Workspace):
                def packages(self):
                    conanfile = self.load_conanfile("dep1")
                    return [{"path": "dep1", "ref": f"{conanfile.name}/{conanfile.version}"}]
            """)

        c.save({"conanws.py": workspace,
                "dep1/conanfile.py": conanfile})
        c.run("workspace info --format=json")
        info = json.loads(c.stdout)
        assert info["packages"] == [{"ref": "pkg/2.1", "path": "dep1"}]
        c.run("install --requires=pkg/2.1")
        # it will not fail

    def test_api_dynamic_any_version(self):
        # For workspace, the version of the conanfile can be ignored if the
        # workspace define a different version.
        c = TestClient(light=True)
        workspace = textwrap.dedent("""\
            import os
            from conan import Workspace

            class MyWorkspace(Workspace):
                def packages(self):
                    return [{"path": "dep1", "ref": "pkg/1.2.3"}]
            """)

        c.save({"conanws.py": workspace,
                "dep1/conanfile.py": GenConanfile("pkg", "0.1")})
        c.run("workspace info --format=json")
        info = json.loads(c.stdout)
        assert info["packages"] == [{"ref": "pkg/1.2.3", "path": "dep1"}]
        c.run("install --requires=pkg/1.2.3")
        # it will not fail

    def test_replace_requires(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "pkga/conanfile.py": GenConanfile("pkga", "0.1"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_requires("pkga/develop"),
                "pkgc/conanfile.py": GenConanfile("pkgc", "0.1").with_requires("pkgb/0.1",
                                                                               "pkga/0.1"),
                "myreplaces": "[replace_requires]\npkga/develop: pkga/0.1"})
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace add pkgc")
        c.run("workspace info --format=json")

        info = json.loads(c.stdout)
        assert info["packages"] == [{'path': 'pkga', 'ref': 'pkga/0.1'},
                                    {'path': 'pkgb', 'ref': 'pkgb/0.1'},
                                    {'path': 'pkgc', 'ref': 'pkgc/0.1'}]
        c.run("install --requires=pkgc/0.1", assert_error=True)
        assert "Version conflict: Conflict between pkga/develop and pkga/0.1 in the graph" in c.out
        # it will not fail
        c.run("install --requires=pkgc/0.1 -pr=default -pr=myreplaces")
        assert "pkga/0.1 - Editable" in c.out
        assert "pkga/develop: pkga/0.1" in c.out

    def test_error_uppercase(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "conanfile.py": GenConanfile("Pkg", "0.1")})
        c.run("workspace add .", assert_error=True)
        assert "ERROR: Conan packages names 'Pkg/0.1' must be all lowercase" in c.out
        c.save({"conanfile.py": GenConanfile()})
        c.run("workspace add . --name=Pkg --version=0.1", assert_error=True)
        assert "ERROR: Conan packages names 'Pkg/0.1' must be all lowercase" in c.out

    def test_add_open_error(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "dep/conanfile.py": GenConanfile("dep", "0.1")})
        c.run("workspace add dep")
        c.run("workspace open dep/0.1", assert_error=True)
        assert "ERROR: Can't open a dependency that is already an editable: dep/0.1" in c.out

    def test_remove_product(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "mydeppkg/conanfile.py": GenConanfile("mydeppkg", "0.1")})
        c.run("workspace add mydeppkg")
        c.run("workspace remove mydeppkg")
        c.run("workspace info")
        assert "mydeppkg" not in c.out

    def test_remove_removed_folder(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "mydeppkg/conanfile.py": GenConanfile("mydeppkg", "0.1")})
        c.run("workspace add mydeppkg")
        # If we now remove the folder
        shutil.rmtree(os.path.join(c.current_folder, "mydeppkg"))
        # It can still be removed by path, even if the path doesn't exist
        c.run("workspace remove mydeppkg")
        assert "Removed from workspace: mydeppkg" in c.out
        c.run("workspace info")
        assert "mydeppkg" not in c.out

    def test_custom_add_remove(self):
        c = TestClient(light=True)

        workspace = textwrap.dedent("""\
            import os
            from conan import Workspace

            class MyWorkspace(Workspace):
                def name(self):
                    return "myws"

                def add(self, ref, path, *args, **kwargs):
                    self.output.info(f"Adding {ref} at {path}")
                    super().add(ref, path, *args, **kwargs)

                def remove(self, path, *args, **kwargs):
                    self.output.info(f"Removing {path}")
                    return super().remove(path, *args, **kwargs)
            """)

        c.save({"conanws.py": workspace,
                "dep/conanfile.py": GenConanfile("dep", "0.1")})
        c.run("workspace add dep")
        assert "Workspace 'myws': Adding dep/0.1" in c.out
        c.run("workspace info")
        assert "dep/0.1" in c.out
        c.run("workspace remove dep")
        assert "Workspace 'myws': Removing" in c.out
        c.run("workspace info")
        assert "dep/0.1" not in c.out

    def test_remove_non_existing_error(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})
        c.run("workspace remove kk", assert_error=True)
        assert "ERROR: No editable package to remove from this path: kk" in c.out


class TestOpenAdd:
    def test_without_git(self):
        t = TestClient(default_server_user=True, light=True)
        t.save({"conanfile.py": GenConanfile("pkg", "0.1")})
        t.run("create .")
        t.run("upload * -r=default -c")

        c = TestClient(servers=t.servers, light=True)
        c.run(f"workspace open pkg/0.1")
        assert "name = 'pkg'" in c.load("pkg/conanfile.py")

        # The add should work the same
        c2 = TestClient(servers=t.servers, light=True)
        c2.save({"conanws.yml": ""})
        c2.run(f"workspace add --ref=pkg/0.1")
        assert "name = 'pkg'" in c2.load("pkg/conanfile.py")
        c2.run("workspace info")
        assert "pkg/0.1" in c2.out

    def test_without_git_export_sources(self):
        t = TestClient(default_server_user=True, light=True)
        t.save({"conanfile.py": GenConanfile("pkg", "0.1").with_exports_sources("*.txt"),
                "CMakeLists.txt": "mycmake"})
        t.run("create .")
        t.run("upload * -r=default -c")

        c = TestClient(servers=t.servers)
        c.run("workspace open pkg/0.1")
        assert "name = 'pkg'" in c.load("pkg/conanfile.py")
        assert "mycmake" in c.load("pkg/CMakeLists.txt")

    def test_workspace_git_scm(self):
        folder = temp_folder()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.scm import Git

            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                def export(self):
                    git = Git(self)
                    git.coordinates_to_conandata()
            """)
        url, commit = create_local_git_repo(files={"conanfile.py": conanfile}, folder=folder,
                                            branch="mybranch")
        t1 = TestClient(default_server_user=True, light=True)
        t1.run_command('git clone "file://{}" .'.format(url))
        t1.run("create .")
        t1.run("upload * -r=default -c")

        c = TestClient(servers=t1.servers, light=True)
        c.run("workspace open pkg/0.1")
        assert c.load("pkg/conanfile.py") == conanfile

        c2 = TestClient(servers=t1.servers, light=True)
        c2.save({"conanws.yml": ""})
        c2.run(f"workspace add --ref=pkg/0.1")
        assert 'name = "pkg"' in c2.load("pkg/conanfile.py")
        c2.run("workspace info")
        assert "pkg/0.1" in c2.out

    def test_workspace_build_editables(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})

        c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_build_msg("BUILD PKGA!"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_build_msg("BUILD PKGB!")
                                                                .with_requires("pkga/0.1")})
        c.run("workspace add pkga")
        c.run("workspace add pkgb")

        c.run("install --requires=pkgb/0.1 --build=editable")
        c.assert_listed_binary({"pkga/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709",
                                             "EditableBuild"),
                                "pkgb/0.1": ("47a5f20ec8fb480e1c5794462089b01a3548fdc5",
                                             "EditableBuild")})
        assert "pkga/0.1: WARN: BUILD PKGA!" in c.out
        assert "pkgb/0.1: WARN: BUILD PKGB!" in c.out


class TestComplete:
    def test_complete_add(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "pkga/conanfile.py": GenConanfile("pkga", "0.1"),
                "pkgx/conanfile.py": GenConanfile("pkgx", "0.1"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_requires("pkga/0.1"),
                "pkgc/conanfile.py": GenConanfile("pkgc", "0.1").with_requires("pkga/0.1",
                                                                               "pkgx/0.1"),
                "pkgd/conanfile.py": GenConanfile("pkgd", "0.1").with_requires("pkgb/0.1",
                                                                               "pkgc/0.1")})

        c.run("workspace complete")  # Does nothing, but it doesn't fail
        assert "There are no packages in this workspace, nothing to complete" in c.out
        c.run("workspace info")
        assert "packages: (empty)" in c.out

        for pkg in ("pkga", "pkgx", "pkgb", "pkgc", "pkgd"):
            c.run(f"export {pkg}")

        c.run("workspace add pkgd")
        c.run("workspace complete")  # Does nothing, but it doesn't fail
        assert "There are no intermediate packages to add to the workspace" in c.out
        c.run("workspace info")
        assert "pkgd/0.1" in c.out
        assert "pkga/0.1" not in c.out

        c.run("workspace add pkga")
        c.run("workspace install --build=missing", assert_error=True)
        assert "Workspace definition error. Package pkgb/0.1 in the Conan cache" in c.out
        c.run("workspace info")
        assert "pkgb/0.1" not in c.out
        assert "pkgc/0.1" not in c.out

        c.run("workspace complete")
        c.run("workspace info")
        assert "pkgb/0.1" in c.out
        assert "pkgc/0.1" in c.out
        assert "pkgx/0.1" not in c.out

    def test_complete_open(self):
        c = TestClient(light=True)
        c.save({"pkgx/conanfile.py": GenConanfile("pkgx", "0.1"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_requires("pkga/0.1"),
                "pkgc/conanfile.py": GenConanfile("pkgc", "0.1").with_requires("pkga/0.1",
                                                                               "pkgx/0.1")})

        for pkg in ("pkgx", "pkgb", "pkgc"):
            c.run(f"export {pkg}")

        c.save({"conanws.yml": "",
                "pkga/conanfile.py": GenConanfile("pkga", "0.1"),
                "pkgd/conanfile.py": GenConanfile("pkgd", "0.1").with_requires("pkgb/0.1",
                                                                               "pkgc/0.1")},
               clean_first=True)

        c.run("workspace add pkgd")
        c.run("workspace add pkga")
        c.run("workspace install --build=missing", assert_error=True)
        assert "Workspace definition error. Package pkgb/0.1 in the Conan cache" in c.out
        c.run("workspace info")
        assert "pkgb/0.1" not in c.out
        assert "pkgc/0.1" not in c.out

        c.run("workspace complete")
        c.run("workspace info")
        assert "pkgb/0.1" in c.out
        assert "pkgc/0.1" in c.out
        assert "pkgx/0.1" not in c.out


class TestWorkspaceBuild:

    def test_build(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})

        c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_build_msg("BUILD PKGA!"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_build_msg("BUILD PKGB!")
               .with_requires("pkga/0.1")})
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace build")
        c.assert_listed_binary({"pkga/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709",
                                             "EditableBuild")})
        assert "conanfile.py (pkga/0.1): WARN: BUILD PKGA!" in c.out
        assert "conanfile.py (pkgb/0.1): WARN: BUILD PKGB!" in c.out

        # It is also possible to build a specific package by path
        # equivalent to ``conan build <path> --build=editable``
        # This can be done even if it is not a product
        c.run("workspace build --pkg=pkgc/*", assert_error=True)
        assert "ERROR: There are no selected packages defined in the workspace" in c.out
        c.run("workspace build --pkg=pkgb/*")
        c.assert_listed_binary({"pkga/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709",
                                             "EditableBuild")})
        assert "conanfile.py (pkga/0.1): WARN: BUILD PKGA!" in c.out
        assert "conanfile.py (pkgb/0.1): WARN: BUILD PKGB!" in c.out

        c.run("workspace build --pkg=pkga/*")
        assert "conanfile.py (pkga/0.1): Calling build()" in c.out
        assert "conanfile.py (pkga/0.1): WARN: BUILD PKGA!" in c.out

    def test_error_if_no_products(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})
        c.run("workspace build", assert_error=True)
        assert "ERROR: There are no selected packages defined in the workspace" in c.out

    def test_build_external_missing(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})

        c.save({"mymath/conanfile.py": GenConanfile("mymath", "0.1").with_build_msg("MyMATH!!"),
                "pkga/conanfile.py": GenConanfile("pkga", "0.1").with_build_msg("BUILD PKGA!")
                                                                .with_requires("mymath/0.1"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_build_msg("BUILD PKGB!")
               .with_requires("pkga/0.1")})
        c.run("export mymath")
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace build", assert_error=True)
        assert "ERROR: Missing prebuilt package for 'mymath/0.1'" in c.out
        c.run("workspace build --build=missing")
        assert "Workspace building external mymath/0.1" in c.out

    def test_build_dynamic_name_version(self):
        conanfile = textwrap.dedent("""\
            from conan import ConanFile
            class MyConan(ConanFile):
                def build(self):
                    self.output.info(f"Building {self.name} AND {self.version}!!!")
            """)
        workspace = textwrap.dedent("""\
            import os
            from conan import Workspace

            class MyWorkspace(Workspace):
                def packages(self):
                    result = []
                    for f in os.listdir(self.folder):
                        if os.path.isdir(os.path.join(self.folder, f)):
                            result.append({"path": f, "ref": f"{f}/0.1"})
                    return result
           """)
        c = TestClient(light=True)
        c.save({"conanws.py": workspace,
                "pkga/conanfile.py": conanfile})
        c.run("workspace info")
        assert "pkga/0.1" in c.out
        c.run("workspace build")
        assert "conanfile.py (pkga/0.1): Building pkga AND 0.1!!!" in c.out

    def test_build_with_external_editable_python_requires(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "ext/conanfile.py": GenConanfile("ext", "0.1").with_package_type("python-require"),
                "pkga/conanfile.py": GenConanfile("pkga", "0.1").with_python_requires("ext/0.1")
                })
        c.run("editable add ext")
        c.run("workspace add pkga")
        c.run("workspace build")
        c.assert_listed_binary({"pkga/0.1": ("8bca2eae7cd0d6b6da8d14f8c86069d89d265bd4",
                                             "EditableBuild")})
        c.assert_listed_require({"ext/0.1": "Editable"}, python=True)

    def test_build_with_external_editable(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": "",
                "ext/conanfile.py": GenConanfile("ext", "0.1").with_build_msg("BUILD EXT!"),
                "pkga/conanfile.py": GenConanfile("pkga", "0.1").with_requires("ext/0.1")
                })
        c.run("editable add ext")
        c.run("workspace add pkga")
        c.run("workspace build")
        assert "ext/0.1: WARN: BUILD EXT!" in c.out
        c.assert_listed_binary({"pkga/0.1": ("38f8a554b1b3c2cbb44321f0e731b3863359126c",
                                             "EditableBuild"),
                                "ext/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709",
                                            "EditableBuild")})


class TestNew:
    def test_new(self):
        # Very basic workspace for testing
        c = TestClient(light=True)
        c.run("new workspace")
        assert 'name = "liba"' in c.load("liba/conanfile.py")
        c.run("workspace info")
        assert "liba/0.1" in c.out
        assert "libb/0.1" in c.out
        assert "app1/0.1" in c.out

    def test_new_dep(self):
        c = TestClient(light=True)
        c.run("new workspace -d requires=dep/0.1")
        assert 'self.requires("dep/0.1")' in c.load("liba/conanfile.py")
        assert 'name = "liba"' in c.load("liba/conanfile.py")
        c.run("workspace info")
        assert "liba/0.1" in c.out
        assert "libb/0.1" in c.out
        assert "app1/0.1" in c.out


class TestMeta:
    def test_install(self):
        c = TestClient()
        c.save({"dep/conanfile.py": GenConanfile()})
        c.run("create dep --name=dep1 --version=0.1")
        c.run("create dep --name=dep2 --version=0.1")
        c.save({"conanws.yml": "",
                "liba/conanfile.py": GenConanfile("liba", "0.1").with_requires("dep1/0.1",
                                                                               "dep2/0.1"),
                "libb/conanfile.py": GenConanfile("libb", "0.1").with_requires("liba/0.1",
                                                                               "dep1/0.1")},
               clean_first=True)
        c.run("workspace add liba")
        c.run("workspace add libb")
        c.run("workspace super-install -g CMakeDeps -g CMakeToolchain -of=build "
              "--envs-generation=false")
        assert "Packages build order:\n    liba/0.1: liba\n    libb/0.1: libb" in c.out
        assert "Workspace conanws.py not found in the workspace folder, using default" in c.out
        files = os.listdir(os.path.join(c.current_folder, "build"))
        assert "conan_toolchain.cmake" in files
        assert "dep1-config.cmake" in files
        assert "dep2-config.cmake" in files
        assert "liba-config.cmake" not in files
        assert "libb-config.cmake" not in files
        assert "conanbuild.bat" not in files
        assert "conanbuild.sh" not in files

    def test_conanfilews_custom(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace

            class MyWs(ConanFile):
                settings = "arch", "build_type"
                generators = "MSBuildDeps"

                def generate(self):
                    self.run("echo myhello world!!!!")

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
                "conanws.py": conanfilews})
        c.run("workspace add dep")
        c.run("workspace super-install -of=build")
        assert "myhello world!!!!" in c.out
        files = os.listdir(os.path.join(c.current_folder, "build"))
        assert "conandeps.props" in files

    def test_conanfilews_errors(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace
            class MyWs(ConanFile):
                requires = "dep/0.1"

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)

        c.save({"conanws.yml": "conanfilews: myconanfilews.py",
                "dep/conanfile.py": GenConanfile("dep", "0.1"),
                "conanws.py": conanfilews})
        c.run("workspace super-install", assert_error=True)
        assert "ERROR: There are no selected packages defined in the workspace" in c.out
        c.run("workspace add dep")
        c.run("workspace super-install", assert_error=True)
        assert "ERROR: Conanfile in conanws.py shouldn't have 'requires'" in c.out

    def test_install_partial(self):
        # If we want to install only some part of the workspace
        c = TestClient()
        c.save({"dep/conanfile.py": GenConanfile()})
        c.run("create dep --name=dep1 --version=0.1")
        c.run("create dep --name=dep2 --version=0.1")
        c.save({"conanws.yml": "",
                "liba/conanfile.py": GenConanfile("liba", "0.1").with_requires("dep1/0.1"),
                "libb/conanfile.py": GenConanfile("libb", "0.1").with_requires("liba/0.1"),
                "libc/conanfile.py": GenConanfile("libc", "0.1").with_requires("libb/0.1",
                                                                               "dep2/0.1")},
               clean_first=True)
        c.run("workspace add liba")
        c.run("workspace add libb")
        c.run("workspace add libc")
        for arg in ("--pkg=libb/*", "--pkg=libb/* --pkg=liba/*"):
            c.run(f"workspace super-install {arg} -g CMakeDeps -of=build")
            assert "dep1/0.1" in c.out
            assert "dep2/0.1" not in c.out
            assert "libc/0.1" not in c.out
            files = os.listdir(os.path.join(c.current_folder, "build"))
            assert "dep1-config.cmake" in files
            assert "dep2-config.cmake" not in files

    def test_workspace_options(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace
            class MyWs(ConanFile):
                settings = "arch", "build_type"
                options = {"myoption": [1, 2, 3]}
                def generate(self):
                    self.output.info(f"Generating with my option {self.options.myoption}!!!!")

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
                "conanws.py": conanfilews})
        c.run("workspace add dep")
        c.run("workspace super-install -of=build")
        assert "conanws.py base project Conanfile: Generating with my option None!!!!" in c.out
        c.run("workspace super-install -of=build -o *:myoption=1")
        assert "conanws.py base project Conanfile: Generating with my option 1!!!!" in c.out

    def test_workspace_common_shared_options(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace
            class MyWs(ConanFile):
                settings = "os"
                options = {"shared": [True, False],
                           "fPIC": [True, False]}
                default_options = {"shared": False, "fPIC": True}
                implements = "auto_shared_fpic"

                def generate(self):
                    self.output.info(f"OS={self.settings.os}!!!!")
                    self.output.info(f"shared={self.options.shared}!!!!")
                    self.output.info(f"fPIC={self.options.get_safe('fPIC')}!!!!")

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
                "conanws.py": conanfilews})
        c.run("workspace add dep")
        c.run("workspace super-install -of=build -s os=Windows")
        assert "conanws.py base project Conanfile: OS=Windows!!!!" in c.out
        assert "conanws.py base project Conanfile: shared=False!!!!" in c.out
        assert "conanws.py base project Conanfile: fPIC=None!!!!" in c.out
        c.run("workspace super-install -of=build -s os=Linux -o *:shared=True")
        assert "conanws.py base project Conanfile: OS=Linux!!!!" in c.out
        assert "conanws.py base project Conanfile: shared=True!!!!" in c.out
        assert "conanws.py base project Conanfile: fPIC=None!!!!" in c.out
        c.run("workspace super-install -of=build -s os=Linux -o *:shared=False")
        assert "conanws.py base project Conanfile: OS=Linux!!!!" in c.out
        assert "conanws.py base project Conanfile: shared=False!!!!" in c.out
        assert "conanws.py base project Conanfile: fPIC=True!!!!" in c.out

    def test_workspace_pkg_options(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace

            class MyWs(ConanFile):
                settings = "arch", "build_type"
                def generate(self):
                    for pkg, dep in self.workspace_packages.items():
                        for k, v in dep.options.items():
                            self.output.info(f"Generating with opt {pkg}:{k}={v}!!!!")

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_option("myoption", [1, 2, 3]),
                "conanws.py": conanfilews})
        c.run("workspace add dep")
        c.run("workspace super-install -of=build")
        assert "project Conanfile: Generating with opt dep/0.1:myoption=None!!!!" in c.out
        c.run("workspace super-install -of=build -o *:myoption=1")
        assert "project Conanfile: Generating with opt dep/0.1:myoption=1!!!!" in c.out

    def test_workspace_pkg_definitions(self):
        c = TestClient()
        conanfilews = textwrap.dedent("""
            from conan import ConanFile
            from conan import Workspace
            from conan.tools.cmake import CMakeToolchain

            class MyWs(ConanFile):
                settings = "arch", "build_type"
                def generate(self):
                    tc = CMakeToolchain(self)
                    for ref, dep in self.workspace_packages.items():
                        dep.configure_toolchain(tc)
                    tc.generate()

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
            """)
        dep = textwrap.dedent("""
            from conan import ConanFile
            class Dep(ConanFile):
                name = "dep"
                version = "1.2.3"

                def configure_toolchain(self, tc):
                    tc.preprocessor_definitions["DEP_SOME_DEFINITION"] = self.version

                def generate(self):
                    tc = CMakeToolchain(self)
                    self.configure_toolchain(tc)
                    tc.generate()
            """)
        pkg = textwrap.dedent("""
            from conan import ConanFile
            class Dep(ConanFile):
                name = "pkg"
                version = "2.3.4"

                def configure_toolchain(self, tc):
                    tc.preprocessor_definitions["SOME_PKG_DEFINE"] = self.version

                def generate(self):
                    tc = CMakeToolchain(self)
                    self.configure_toolchain(tc)
                    tc.generate()
            """)
        c.save({"dep/conanfile.py": dep,
                "pkg/conanfile.py": pkg,
                "conanws.py": conanfilews})
        c.run("workspace add dep")
        c.run("workspace add pkg")
        c.run("workspace super-install")
        tc = c.load("conan_toolchain.cmake")
        assert 'add_compile_definitions("SOME_PKG_DEFINE=2.3.4")' in tc
        assert 'add_compile_definitions("DEP_SOME_DEFINITION=1.2.3")' in tc

    def test_intermediate_non_editable(self):
        c = TestClient(light=True)

        c.save({"liba/conanfile.py": GenConanfile("liba", "0.1"),
                "libb/conanfile.py": GenConanfile("libb", "0.1").with_requires("liba/0.1"),
                "libc/conanfile.py": GenConanfile("libc", "0.1").with_requires("libb/0.1")})

        c.run("workspace init")
        c.run("workspace add liba")
        c.run("export libb")
        c.run("workspace add libc")
        c.run("workspace super-install", assert_error=True)
        assert ("Workspace definition error. Package libb/0.1 in the Conan cache "
                "has dependencies to packages in the workspace: [liba/0.1]") in c.out

    def test_deployers_json(self):
        c = TestClient()
        dep = textwrap.dedent("""
            import os
            from conan.tools.files import save
            from conan import ConanFile
            class Dep(ConanFile):
                name = "dep"
                version = "1.0"

                def package(self):
                    save(self, os.path.join(self.package_folder, "myfile.txt"), "content")
            """)
        pkg = textwrap.dedent("""
            import os
            from conan.tools.files import save
            from conan import ConanFile
            class Dep(ConanFile):
                name = "pkg"
                version = "1.0"
                requires = "dep/1.0"

                def package(self):
                    save(self, os.path.join(self.package_folder, "myfile.txt"), "content")
            """)
        c.save({"dep/conanfile.py": dep,
                "pkg/conanfile.py": pkg})
        c.run("workspace init")
        c.run("create dep")
        c.run("workspace add pkg")
        c.run("workspace super-install --format=json --deployer=full_deploy")
        graph = json.loads(c.stdout)
        # Only 1 node, pkg is not a node in the graph!
        assert graph["graph"]["nodes"]["1"]["name"] == "dep"
        deploy_folder = os.path.join(c.current_folder, "full_deploy", "host")
        folders = os.listdir(deploy_folder)
        assert "dep" in folders
        assert "pkg" not in folders
        assert os.path.isfile(os.path.join(deploy_folder, "dep", "1.0", "myfile.txt"))

    def test_customize_generator_per_package(self):
        # https://github.com/conan-io/conan/issues/19326
        c = TestClient(light=True)

        pkg = GenConanfile("pkg", "0.1")
        workspace = textwrap.dedent("""\
            from conan import Workspace, ConanFile

            class MyWs(ConanFile):
                pass

            class Ws(Workspace):
                def root_conanfile(self):
                    return MyWs
           """)
        c.save({"pkg/conanfile.py": pkg,
                "conanws.py": workspace})
        c.run("workspace add pkg")
        c.run("workspace super-install -c acme*:tools.cmake.cmaketoolchain:generator=Ninja")
        # It no longer crashes
        assert "Install finished successfully" in c.out


def test_workspace_with_local_recipes_index():
    c3i_folder = temp_folder()
    recipes_folder = os.path.join(c3i_folder, "recipes")
    zlib_config = textwrap.dedent("""
       versions:
         "1.2.11":
           folder: all
       """)
    save_files(recipes_folder, {"zlib/config.yml": zlib_config,
                                "zlib/all/conanfile.py": str(GenConanfile("zlib")),
                                "zlib/all/conandata.yml": ""})

    c = TestClient(light=True)
    c.save({".conanrc": 'conan_home=deps\n'})
    c.run(f'remote add local "{c3i_folder}"')

    c.run("list zlib/1.2.11#* -r=local")
    assert "zlib/1.2.11" in c.out  # It doesn't crash
    c.run("list zlib/1.2.11#*")
    assert "zlib/1.2.11" not in c.out


class TestClean:
    def test_clean(self):
        # Using cmake_layout, we can clean the build folders
        c = TestClient()
        c.save({"conanws.yml": "name: my_workspace"})
        pkga = GenConanfile("pkga", "0.1").with_settings("build_type")
        pkgb = GenConanfile("pkgb", "0.1").with_requires("pkga/0.1").with_settings("build_type")
        c.save({"pkga/conanfile.py": pkga,
                "pkgb/conanfile.py": pkgb,
                "pkgc/conanfile.py": GenConanfile("pkgc", "0.1").with_requires("pkgb/0.1")})
        c.run("workspace add pkga -of=build/pkga")
        c.run("workspace add pkgb -of=build/pkgb")
        c.run("workspace add pkgc")
        c.run("workspace build")
        assert os.path.exists(os.path.join(c.current_folder, "build", "pkga"))
        assert os.path.exists(os.path.join(c.current_folder, "build", "pkgb"))
        c.run("workspace clean")
        assert "Workspace 'my_workspace': Removing pkga/0.1 output folder" in c.out
        assert "Workspace 'my_workspace': Removing pkgb/0.1 output folder" in c.out
        assert "Editable pkgc/0.1 doesn't have an output_folder defined" in c.out
        assert not os.path.exists(os.path.join(c.current_folder, "build", "pkga"))
        assert not os.path.exists(os.path.join(c.current_folder, "build", "pkgb"))

    def test_custom_clean(self):
        conanfilews = textwrap.dedent("""
            from conan import Workspace

            class Ws(Workspace):
                def name(self):
                    return "my_workspace"
                def clean(self):
                    self.output.info("MY CLEAN!!!!")
            """)
        c = TestClient()
        c.save({"conanws.py": conanfilews})
        c.run("workspace clean")
        assert "Workspace 'my_workspace': MY CLEAN!!!" in c.out


def test_relative_paths():
    # This is using the meta-project
    c = TestClient()
    c.run("new cmake_lib -d name=liba --output=liba")
    c.run("new cmake_exe -d name=app1 -d requires=liba/0.1 --output=app1")
    c.run("new cmake_lib -d name=libb --output=other/libb")
    c.run("new cmake_exe -d name=app2 -d requires=libb/0.1 --output=other/app2")
    c.run("workspace init mywks")
    c.run("workspace init otherwks")
    # cd mywks
    with c.chdir("mywks"):
        c.run("workspace add ../liba")
        c.run("workspace add ../app1")
        c.run("workspace info")
        expected = textwrap.dedent("""\
        packages
          - path: ../liba
            ref: liba/0.1
          - path: ../app1
            ref: app1/0.1
        """)
        assert expected in c.out
        c.run("graph info --requires=app1/0.1")
        c.assert_listed_require({"app1/0.1": "Editable", "liba/0.1": "Editable"})
    # cd otherwks
    with c.chdir("otherwks"):
        c.run("workspace add ../other/libb")
        c.run("workspace add ../other/app2")
        c.run("workspace info")
        expected = textwrap.dedent("""\
        packages
          - path: ../other/libb
            ref: libb/0.1
          - path: ../other/app2
            ref: app2/0.1
        """)
        assert expected in c.out
        c.run("graph info --requires=app2/0.1")
        c.assert_listed_require({"app2/0.1": "Editable", "libb/0.1": "Editable"})


def test_host_build_require():
    c = TestClient()
    protobuf = textwrap.dedent("""\
        from conan import ConanFile
        from conan.tools.files import save, copy
        class Protobuf(ConanFile):
            name = "protobuf"
            version = "0.1"
            settings = "os"

            def layout(self):
                self.folders.build = f"mybuild/folder_{self.settings.os}"
                self.cpp.build.bindirs = ["."]

            def build(self):
                save(self, "myprotobuf.txt", f"MYOS={self.settings.os}!!!")

            def package(self):
                copy(self, "myprotobuf.txt", src=self.build_folder, dst=self.package_folder)

            def package_info(self):
                self.cpp_info.bindirs = ["."]
        """)
    app = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools.files import load
        class Protobuf(ConanFile):
            name = "app"
            version = "0.1"
            requires = "protobuf/0.1"
            tool_requires = "protobuf/0.1"

            def build(self):
                binhost = self.dependencies.host["protobuf"].cpp_info.bindir
                binbuild = self.dependencies.build["protobuf"].cpp_info.bindir
                host = load(self, os.path.join(binhost, "myprotobuf.txt"))
                build = load(self, os.path.join(binbuild, "myprotobuf.txt"))
                self.output.info(f"BUILDING WITH BINHOST: {host}")
                self.output.info(f"BUILDING WITH BINBUILD: {build}")
        """)
    c.save({"protobuf/conanfile.py": protobuf,
            "app/conanfile.py": app})
    c.run("workspace init .")
    c.run("workspace add protobuf")
    c.run("workspace add app")
    c.run("install app --build=editable -s:b os=Linux -s:h os=Windows")

    assert c.load("protobuf/mybuild/folder_Linux/myprotobuf.txt") == "MYOS=Linux!!!"
    assert c.load("protobuf/mybuild/folder_Windows/myprotobuf.txt") == "MYOS=Windows!!!"

    shutil.rmtree(os.path.join(c.current_folder, "protobuf", "mybuild"))
    assert not os.path.exists(os.path.join(c.current_folder, "protobuf", "mybuild"))
    c.run("workspace build -s:b os=Linux -s:h os=Windows")
    assert "conanfile.py (app/0.1): BUILDING WITH BINHOST: MYOS=Windows!!!" in c.out
    assert "conanfile.py (app/0.1): BUILDING WITH BINBUILD: MYOS=Linux!!!" in c.out
    assert c.load("protobuf/mybuild/folder_Linux/myprotobuf.txt") == "MYOS=Linux!!!"
    assert c.load("protobuf/mybuild/folder_Windows/myprotobuf.txt") == "MYOS=Windows!!!"

    shutil.rmtree(os.path.join(c.current_folder, "protobuf", "mybuild"))
    assert not os.path.exists(os.path.join(c.current_folder, "protobuf", "mybuild"))
    c.run("workspace create -s:b os=Linux -s:h os=Windows")
    assert "app/0.1: BUILDING WITH BINHOST: MYOS=Windows!!!" in c.out
    assert "app/0.1: BUILDING WITH BINBUILD: MYOS=Linux!!!" in c.out


class TestCreate:
    def test_create(self):
        c = TestClient(light=True, default_server_user=True)
        c.save({"conanfile.py": GenConanfile("zlib", "0.1").with_build_msg("BUILD ZLIB!")})
        c.run("export .")
        c.save({"conanws.yml": ""}, clean_first=True)
        c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_requires("zlib/0.1")
                                                                .with_build_msg("BUILD PKGA!"),
                "pkga/test_package/conanfile.py": GenConanfile().with_test("pass"),
                "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_build_msg("BUILD PKGB!")
               .with_requires("pkga/0.1"),
                "pkgb/test_package/conanfile.py": GenConanfile().with_test("pass"),
                "pkgc/conanfile.py": GenConanfile("pkgc", "0.1").with_build_msg("BUILD PKGC!")
               .with_requires("pkgb/0.1"),
                "pkgc/test_package/conanfile.py": GenConanfile().with_test("pass"),
                })
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace add pkgc")
        c.run("workspace create --build=zlib/*")
        assert str(c.out).count("zlib/0.1: WARN: BUILD ZLIB!") == 1
        assert str(c.out).count("pkga/0.1: WARN: BUILD PKGA!") == 1
        assert str(c.out).count("pkgb/0.1: WARN: BUILD PKGB!") == 1
        assert str(c.out).count("pkgc/0.1: WARN: BUILD PKGC!") == 1
        assert "pkga/0.1 (test package): Running test()" in c.out
        assert "pkgb/0.1 (test package): Running test()" in c.out
        assert "pkgc/0.1 (test package): Running test()" in c.out

        # A second conan workspace create will not re-build
        c.run("workspace create")
        assert "WARN: BUILD ZLIB!" not in c.out
        assert "WARN: BUILD PKGA!" not in c.out
        assert "WARN: BUILD PKGB!" not in c.out
        assert "WARN: BUILD PKBC!" not in c.out
        assert "pkga/0.1 (test package): Running test()" not in c.out
        assert "pkgb/0.1 (test package): Running test()" not in c.out
        assert "pkgc/0.1 (test package): Running test()" not in c.out

        c.run("upload zlib/* -r=default -c")
        c.run("remove zlib/* -c")
        c.run("remove pkgc/* -c")
        c.run("workspace create")
        assert "WARN: BUILD ZLIB!" not in c.out
        assert "pkgc/0.1: WARN: BUILD PKGC!" in c.out

    def test_create_dynamic_name(self):
        workspace = textwrap.dedent("""\
            import os
            from conan import Workspace

            class MyWorkspace(Workspace):
                def packages(self):
                    result = []
                    for f in os.listdir(self.folder):
                        if os.path.isdir(os.path.join(self.folder, f)):
                            result.append({"path": f, "ref": f"{f}/0.1"})
                    return result
           """)
        c = TestClient(light=True)
        c.save({"conanws.py": workspace,
                "pkga/conanfile.py": GenConanfile()})
        c.run("workspace info")
        assert "pkga/0.1" in c.out
        c.run("workspace create")
        assert "Workspace create pkga/0.1" in c.out

    def test_host_build_require(self):
        c = TestClient()
        protobuf = textwrap.dedent("""\
            from conan import ConanFile
            class Protobuf(ConanFile):
                name = "protobuf"
                version = "0.1"
                settings = "os"

                def build(self):
                    self.output.info(f"Building for: {self.settings.os}!!!")
            """)
        app = textwrap.dedent("""\
            from conan import ConanFile
            class Protobuf(ConanFile):
                name = "app"
                version = "0.1"
                requires = "protobuf/0.1"
                tool_requires = "protobuf/0.1"
            """)
        c.save({"protobuf/conanfile.py": protobuf,
                "app/conanfile.py": app})
        c.run("workspace init .")
        c.run("workspace add protobuf")
        c.run("workspace add app")

        c.run("workspace create -s:b os=Linux -s:h os=Windows")

        assert "protobuf/0.1: Building for: Windows!!!" in c.out
        assert "protobuf/0.1: Building for: Linux!!!" in c.out


class TestSource:
    def test_source(self):
        c = TestClient(light=True)
        c.save({"conanws.yml": ""})

        pkga = textwrap.dedent("""\
            from conan import ConanFile
            class TestPackage(ConanFile):
                name = "pkga"
                version = "0.1"
                def source(self):
                    self.output.info("Executing SOURCE!!!")
            """)
        pkgb = textwrap.dedent("""\
            from conan import ConanFile
            class TestPackage(ConanFile):
                name = "pkgb"
                version = "0.1"
                requires = "pkga/0.1"
                def source(self):
                    self.output.info("Executing SOURCE!!!")
            """)
        pkgc = textwrap.dedent("""\
            from conan import ConanFile
            class TestPackage(ConanFile):
                name = "pkgc"
                version = "0.1"
                requires = "pkgb/0.1"
                def source(self):
                    self.output.info("Executing SOURCE!!!")
            """)
        c.save({"pkga/conanfile.py": pkga,
                "pkgb/conanfile.py": pkgb,
                "pkgc/conanfile.py": pkgc})
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace add pkgc")
        c.run("workspace source")
        assert "conanfile.py (pkga/0.1): Executing SOURCE!!!" in c.out
        assert "conanfile.py (pkgb/0.1): Executing SOURCE!!!" in c.out
        assert "conanfile.py (pkgc/0.1): Executing SOURCE!!!" in c.out


class TestInstall:
    def test_install(self):
        c = TestClient()
        c.save({"conanws.yml": ""})

        pkg = textwrap.dedent("""\
            from conan import ConanFile
            class Package(ConanFile):
                name = "{}"
                version = "0.1"
                settings = "os", "build_type"
                {}
                generators = "CMakeToolchain"
            """)

        c.save({"pkga/conanfile.py": pkg.format("pkga", ''),
                "pkgb/conanfile.py": pkg.format("pkgb", 'requires="pkga/0.1"'),
                "pkgc/conanfile.py": pkg.format("pkgc", 'requires="pkgb/0.1"')})
        c.run("workspace add pkga")
        c.run("workspace add pkgb")
        c.run("workspace add pkgc")
        c.run("workspace install")
        assert "conanfile.py (pkga/0.1): CMakeToolchain generated" in c.out
        assert "conanfile.py (pkgb/0.1): CMakeToolchain generated" in c.out
        assert "conanfile.py (pkgc/0.1): CMakeToolchain generated" in c.out


def test_keep_core_conf():
    c = TestClient()
    myprofile = textwrap.dedent("""\
        [settings]
        os=FreeBSD
        arch=armv7
        """)
    conanfile = textwrap.dedent("""\
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkga"
            version = "0.1"
            settings = "os", "arch"

            def generate(self):
                self.output.info(f"Generating!: {self.settings.os}-{self.settings.arch}!!!!")
        """)
    c.save_home({"profiles/myprofile": myprofile})
    c.run("profile show ")
    c.save({"conanws.yml": "",
            "pkga/conanfile.py": conanfile})
    c.run("workspace add pkga")
    # The injected -cc is still applied to every "conan install"
    c.run("workspace install -cc core:default_profile=myprofile")
    assert "conanfile.py (pkga/0.1): Generating!: FreeBSD-armv7!!!!" in c.out
    # also the global.conf
    c.save_home({"global.conf": "core:default_profile=myprofile"})
    c.run("workspace install")
    assert "conanfile.py (pkga/0.1): Generating!: FreeBSD-armv7!!!!" in c.out


def test_workspace_defining_only_paths():
    c = TestClient()
    c.run("new workspace")
    conanws_with_labels = textwrap.dedent("""\
    packages:
      - path: liba
      - path: libb
      - path: app1
    """)
    c.save({"conanws.yml": conanws_with_labels})
    # liba with user and channel too
    replace_in_file(ConanFileMock(), os.path.join(c.current_folder, "liba", "conanfile.py"),
                    'version = "0.1"',
                    'version = "0.1"\n    user = "myuser"\n    channel = "mychannel"')
    replace_in_file(ConanFileMock(), os.path.join(c.current_folder, "libb", "conanfile.py"),
                    'self.requires("liba/0.1")',
                    'self.requires("liba/0.1@myuser/mychannel")')
    c.run("workspace info")
    expected = textwrap.dedent("""\
    packages
      - path: liba
      - path: libb
      - path: app1
    """)
    assert expected in c.out
    c.run("workspace install")
    assert "conanfile.py (app1/0.1)" in c.out
    assert "liba/0.1@myuser/mychannel - Editable" in c.out
    assert "libb/0.1 - Editable" in c.out

    c.run("workspace super-install")
    assert "app1/0.1 - Editable" in c.out
    assert "liba/0.1@myuser/mychannel - Editable" in c.out
    assert "libb/0.1 - Editable" in c.out


def test_workspace_defining_duplicate_references():
    """
    Testing duplicate references but different paths
    """
    c = TestClient(light=True)
    conanws_with_labels = textwrap.dedent("""\
    packages:
      - path: liba
      - path: liba1
      - path: liba2
    """)
    c.save({
        "conanws.yml": conanws_with_labels,
        "liba/conanfile.py": GenConanfile(name="liba", version="0.1"),
        "liba1/conanfile.py": GenConanfile(name="liba", version="0.1"),
        "liba2/conanfile.py": GenConanfile(name="liba", version="0.1"),
    })
    c.run("workspace install", assert_error=True)
    assert "Workspace package 'liba/0.1' already exists." in c.out


def test_workspace_reference_error():
    c = TestClient(light=True)
    conanws_with_labels = textwrap.dedent("""\
    packages:
      - path: libx
    """)
    c.save({"conanws.yml": conanws_with_labels,
            "libx/conanfile.py": ""})
    c.run("workspace install", assert_error=True)
    assert ("Workspace package reference could not be deduced by libx/conanfile.py or it is not"
            " correctly defined in the conanws.yml file") in c.out


def test_workspace_python_error():
    c = TestClient()
    workspace = textwrap.dedent("""\
       from conan import Workspace

       class MyWorkspace(Workspace):
           def packages(self):
               os.listdir(self.folder)
      """)
    c.save({"conanws.yml": "",
            "conanws.py": "bad"})
    c.run("workspace info", assert_error=True)
    assert "ERROR: Error loading conanws.py at" in c.out
    c.save({"conanws.yml": "",
            "conanws.py": workspace})
    c.run("workspace info", assert_error=True)
    assert "ERROR: Workspace conanws.py file: Error in packages() method, line 5" in c.out



import platform
import textwrap

import pytest

from conan.test.assets.sources import gen_function_cpp
from conan.test.utils.tools import TestClient


@pytest.fixture
def client():
    lib_conanfile = textwrap.dedent("""
        from conan import ConanFile

        class FooLib(ConanFile):
            name = "foolib"
            version = "1.0"

            def package_info(self):
                self.cpp_info.frameworks.extend(['Foundation', 'CoreServices', 'CoreFoundation'])
    """)

    t = TestClient()
    t.save({'conanfile.py': lib_conanfile})
    t.run("create .")
    return t


app_conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.cmake import CMake

    class App(ConanFile):
        requires = "foolib/1.0"
        generators = "CMakeDeps", "CMakeToolchain"
        settings = "build_type", "os", "arch"

        def build(self):
            cmake = CMake(self)
            cmake.configure()
""")


# needs at least 3.23.3 because of error with "empty identity"
# https://stackoverflow.com/questions/72746725/xcode-14-beta-cmake-not-able-to-resolve-cmake-c-compiler-and-cmake-cxx-compiler
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.tool("cmake", "3.23")
def test_apple_framework_xcode(client):
    app_cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(Testing NONE)
        find_package(foolib REQUIRED)
        message(">>> foolib_FRAMEWORKS_FOUND_DEBUG: ${foolib_FRAMEWORKS_FOUND_DEBUG}")
        message(">>> foolib_FRAMEWORKS_FOUND_RELEASE: ${foolib_FRAMEWORKS_FOUND_RELEASE}")
    """)

    client.save({'conanfile.py': app_conanfile,
                 'CMakeLists.txt': app_cmakelists})

    client.run("build . -c tools.cmake.cmaketoolchain:generator=Xcode")
    assert "/System/Library/Frameworks/Foundation.framework;" in client.out
    assert "/System/Library/Frameworks/CoreServices.framework;" in client.out
    assert "/System/Library/Frameworks/CoreFoundation.framework" in client.out


conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.cmake import CMake, CMakeToolchain

            class AppleframeworkConan(ConanFile):
                settings = "os", "compiler", "build_type", "arch"
                generators = "CMakeDeps", "CMakeToolchain"
                exports_sources = "src/*"
                name = "mylibrary"
                version = "1.0"

                def layout(self):
                    self.folders.source = "src"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                    cmake.install()
                    self.run("otool -L '%s/hello.framework/hello'" % self.build_folder)
                    self.run("otool -L '%s/hello.framework/hello'" % self.package_folder)

                def package_info(self):
                    self.cpp_info.frameworkdirs.append(self.package_folder)
                    self.cpp_info.frameworks.append("hello")
                    self.cpp_info.includedirs = []
            """)
cmake = textwrap.dedent("""
            set(CMAKE_CXX_COMPILER_WORKS 1)
            set(CMAKE_CXX_ABI_COMPILED 1)
            cmake_minimum_required(VERSION 3.15)
            project(MyHello CXX)

            # set @rpaths for libraries to link against
            SET(CMAKE_SKIP_RPATH FALSE)
            #SET(CMAKE_SKIP_BUILD_RPATH  FALSE)
            #SET(CMAKE_INSTALL_RPATH "@rpath/")
            #SET(CMAKE_INSTALL_RPATH_USE_LINK_PATH TRUE)

            add_library(hello SHARED hello.cpp hello.h)
            set_target_properties(hello PROPERTIES
              FRAMEWORK TRUE
              FRAMEWORK_VERSION A
              MACOSX_FRAMEWORK_IDENTIFIER com.cmake.hello
              MACOSX_FRAMEWORK_INFO_PLIST src/Info.plist
              # "current version" in semantic format in Mach-O binary file
              VERSION 1.6.0
              # "compatibility version" in semantic format in Mach-O binary file
              SOVERSION 1.6.0
              PUBLIC_HEADER hello.h
              INSTALL_NAME_DIR "@rpath"
              MACOSX_RPATH TRUE
            )
            install(TARGETS hello DESTINATION ".")
        """)
hello_h = textwrap.dedent("""
            #pragma once

            #ifdef WIN32
              #define HELLO_EXPORT __declspec(dllexport)
            #else
              #define HELLO_EXPORT __attribute__((visibility("default")))
            #endif

            #ifdef __cplusplus
            extern "C" {
            #endif
            class HELLO_EXPORT Hello
            {
                public:
                    static void hello();
            };
            #ifdef __cplusplus
            }
            #endif
        """)
hello_cpp = textwrap.dedent("""
            #include <iostream>
            #include "hello.h"

            void Hello::hello(){
                #ifdef NDEBUG
                std::cout << "Hello World Release!" <<std::endl;
                #else
                std::cout << "Hello World Debug!" <<std::endl;
                #endif
            }
        """)
infoplist = textwrap.dedent("""
            <?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN"
                     "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
            <plist version="1.0">
            <dict>
                <key>CFBundleDisplayName</key>
                <string>hello</string>
                <key>CFBundleExecutable</key>
                <string>hello</string>
                <key>CFBundleIdentifier</key>
                <string>com.test.hello</string>
                <key>CFBundleInfoDictionaryVersion</key>
                <string>6.0</string>
                <key>CFBundleName</key>
                <string>hello</string>
                <key>CFBundlePackageType</key>
                <string>FMWK</string>
                <key>CFBundleShortVersionString</key>
                <string>1.6.0</string>
                <key>CFBundleVersion</key>
                <string>1.6.0</string>
                <key>Flavor_ID</key>
                <string>0</string>
                <key>NSAppTransportSecurity</key>
                <dict>
                    <key>NSAllowsArbitraryLoads</key>
                    <true/>
                </dict>
                <key>NSPrincipalClass</key>
                <string></string>
            </dict>
            </plist>
        """)
timer_cpp = textwrap.dedent("""
    #include <hello/hello.h>
    int main(){
        Hello::hello();
    }
    """)


@pytest.mark.tool("cmake")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("settings",
         [('',),
          ('-pr:b default -s os=iOS -s os.sdk=iphoneos -s os.version=10.0 -s arch=armv8',),
          ("-pr:b default -s os=tvOS -s os.sdk=appletvos -s os.version=11.0 -s arch=armv8",)])
def test_apple_own_framework_cross_build(settings):
    client = TestClient()

    test_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(Testing CXX)

        find_package(mylibrary REQUIRED)

        add_executable(timer timer.cpp)
        target_link_libraries(timer mylibrary::mylibrary)
    """)

    test_conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMake, CMakeToolchain, CMakeDeps, cmake_layout
        from conan.tools.build import cross_building

        class TestPkg(ConanFile):
            generators = "CMakeToolchain"
            settings = "os", "arch", "compiler", "build_type"

            def layout(self):
                cmake_layout(self)

            def requirements(self):
                self.requires(self.tested_reference_str)
                self.tool_requires(self.tested_reference_str)

            def generate(self):
                cmake = CMakeDeps(self)
                cmake.build_context_activated = ["mylibrary"]
                cmake.build_context_suffix = {"mylibrary": "_BUILD"}
                cmake.generate()

            def build(self):
                self.output.warning("Building test package at: {}".format(self.build_folder))
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def test(self):
                if not cross_building(self):
                    cmd = os.path.join(self.cpp.build.bindirs[0], "timer")
                    self.run(cmd, env="conanrunenv")
        """)

    client.save({'conanfile.py': conanfile,
                 "src/CMakeLists.txt": cmake,
                 "src/hello.h": hello_h,
                 "src/hello.cpp": hello_cpp,
                 "src/Info.plist": infoplist,
                 "test_package/conanfile.py": test_conanfile,
                 'test_package/CMakeLists.txt': test_cmake,
                 "test_package/timer.cpp": timer_cpp})
    # First build it as build_require in the build-context, no testing
    # the UX could be improved, but the simplest could be:
    #  - Have users 2 test_packages, one for the host and other for the build, with some naming
    #    convention. CI launches one after the other if found
    client.run("create . %s -tf=\"\" --build-require" % settings)
    client.run("create . %s" % settings)
    if not len(settings):
        assert "Hello World Release!" in client.out


@pytest.mark.xfail(reason="run_environment=True no longer works")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.tool("cmake", "3.19")
def test_apple_own_framework_cmake_deps():
    client = TestClient()

    test_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(Testing CXX)
        message(STATUS "CMAKE_BINARY_DIR ${CMAKE_BINARY_DIR}")
        find_package(mylibrary REQUIRED)
        message(">>> MYLIBRARY_FRAMEWORKS_FOUND_DEBUG: ${MYLIBRARY_FRAMEWORKS_FOUND_DEBUG}")
        message(">>> MYLIBRARY_FRAMEWORKS_FOUND_RELEASE: ${MYLIBRARY_FRAMEWORKS_FOUND_RELEASE}")
        add_executable(timer timer.cpp)
        target_link_libraries(timer mylibrary::mylibrary)
    """)

    test_conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMake

        class TestPkg(ConanFile):
            generators = "CMakeToolchain"
            name = "app"
            version = "1.0"
            requires = "mylibrary/1.0"
            exports_sources = "CMakeLists.txt", "timer.cpp"
            settings = "os", "arch", "compiler", "build_type"

            def requirements(self):
                self.tool_requires(self.tested_reference_str)

            def generate(self):
                cmake = CMakeDeps(self)
                cmake.build_context_activated = ["mylibrary"]
                cmake.build_context_suffix = {"mylibrary": "_BUILD"}
                cmake.generate()

            def layout(self):
                self.folders.build = str(self.settings.build_type)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def test(self):
                self.run(os.path.join(str(self.settings.build_type), "timer"), env="conanrunenv")
        """)
    client.save({'conanfile.py': conanfile,
                 "src/CMakeLists.txt": cmake,
                 "src/hello.h": hello_h,
                 "src/hello.cpp": hello_cpp,
                 "src/Info.plist": infoplist})
    client.run("export . --name=mylibrary --version=1.0")
    client.run("create . --name=mylibrary --version=1.0 -s build_type=Debug")
    client.run("create . --name=mylibrary --version=1.0 -s build_type=Release")

    profile = textwrap.dedent("""
        include(default)
        [conf]
        tools.cmake.cmaketoolchain:generator=Xcode
        """)
    client.save({"conanfile.py": test_conanfile,
                 'CMakeLists.txt': test_cmake,
                 "timer.cpp": timer_cpp,
                 "profile": profile})

    client.run("install . -s build_type=Debug -pr=profile")
    client.run("install . -s build_type=Release -pr=profile")
    client.run("test . mylibrary/1.0@  -pr=profile")
    assert "Hello World Release!" in client.out
    client.run("test . mylibrary/1.0@ -s:b build_type=Debug  -pr=profile")
    assert "Hello World Debug!" in client.out


@pytest.mark.tool("cmake")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
def test_apple_own_framework_cmake_find_package_multi():
    client = TestClient()

    test_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(Testing CXX)
        set(CMAKE_RUNTIME_OUTPUT_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}/bin")
        set(CMAKE_RUNTIME_OUTPUT_DIRECTORY_DEBUG "${CMAKE_CURRENT_BINARY_DIR}/bin")
        set(CMAKE_RUNTIME_OUTPUT_DIRECTORY_RELEASE "${CMAKE_CURRENT_BINARY_DIR}/bin")
        find_package(mylibrary REQUIRED)
        message(">>> CONAN_FRAMEWORKS_FOUND_MYLIBRARY: ${CONAN_FRAMEWORKS_FOUND_MYLIBRARY}")
        add_executable(timer timer.cpp)
        target_link_libraries(timer mylibrary::mylibrary)
    """)

    test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake
        class TestPkg(ConanFile):
            generators = "CMakeDeps", "CMakeToolchain"
            settings = "build_type", "os", "arch"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
            def test(self):
                self.run("bin/timer", env="conanrunenv")
        """)
    client.save({'conanfile.py': conanfile,
                 "src/CMakeLists.txt": cmake,
                 "src/hello.h": hello_h,
                 "src/hello.cpp": hello_cpp,
                 "src/Info.plist": infoplist,
                 "test_package/conanfile.py": test_conanfile,
                 'test_package/CMakeLists.txt': test_cmake,
                 "test_package/timer.cpp": timer_cpp})
    client.run("create .")
    assert "Hello World Release!" in client.out


@pytest.mark.tool("cmake", "3.19")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
def test_component_uses_apple_framework():
    conanfile_py = textwrap.dedent("""
from conan import ConanFile, tools
from conan.tools.cmake import CMake


class HelloConan(ConanFile):
    name = "hello"
    description = "example"
    topics = ("conan",)
    url = "https://github.com/conan-io/conan-center-index"
    homepage = "https://www.example.com"
    license = "MIT"
    exports_sources = ["hello.cpp", "hello.h", "CMakeLists.txt"]
    generators = "CMakeDeps", "CMakeToolchain"
    settings = "os", "arch", "compiler", "build_type"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
        cmake.install()

    def package_info(self):
        self.cpp_info.set_property("cmake_file_name", "HELLO")
        self.cpp_info.components["libhello"].set_property("cmake_target_name", "hello::libhello")
        self.cpp_info.components["libhello"].libs = ["hello"]
        # We need to add the information about the lib/include directories to be able to find them
        self.cpp_info.components["libhello"].libdirs = ["lib"]
        self.cpp_info.components["libhello"].includedirs = ["include"]

        self.cpp_info.components["libhello"].frameworks.extend(["CoreFoundation"])
        """)
    hello_cpp = textwrap.dedent("""
#include <CoreFoundation/CoreFoundation.h>

void hello_api()
{
    CFTypeRef keys[] = {CFSTR("key")};
    CFTypeRef values[] = {CFSTR("value")};
    CFDictionaryRef dict = CFDictionaryCreate(kCFAllocatorDefault, keys, values, sizeof(keys) / sizeof(keys[0]), &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    if (dict)
        CFRelease(dict);
}
        """)
    hello_h = textwrap.dedent("""
void hello_api();
        """)
    cmakelists_txt = textwrap.dedent("""
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
cmake_minimum_required(VERSION 3.15)
project(hello)
include(GNUInstallDirs)
file(GLOB SOURCES *.cpp)
file(GLOB HEADERS *.h)
add_library(${PROJECT_NAME} ${SOURCES} ${HEADERS})
set_target_properties(${PROJECT_NAME} PROPERTIES PUBLIC_HEADER ${HEADERS})
install(TARGETS ${PROJECT_NAME}
    RUNTIME DESTINATION bin
    LIBRARY DESTINATION lib
    ARCHIVE DESTINATION lib
    PUBLIC_HEADER DESTINATION include)
        """)
    test_conanfile_py = textwrap.dedent("""
import os
from conan import ConanFile, tools
from conan.tools.cmake import CMake, CMakeToolchain, CMakeDeps
from conan.tools.build import cross_building


class TestPackageConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "CMakeToolchain"

    def requirements(self):
        self.requires(self.tested_reference_str)
        self.tool_requires(self.tested_reference_str)

    def generate(self):
        cmake = CMakeDeps(self)
        cmake.build_context_activated = ["hello"]
        cmake.build_context_suffix = {"hello": "_BUILD"}
        cmake.generate()

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def test(self):
        if not cross_building(self):
            self.run("./test_package", env="conanrunenv")
        """)
    test_test_package_cpp = textwrap.dedent("""
#include "hello.h"

int main()
{
    hello_api();
}
        """)
    test_cmakelists_txt = textwrap.dedent("""
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
cmake_minimum_required(VERSION 3.15)
project(test_package CXX)

find_package(HELLO REQUIRED CONFIG)

add_executable(${PROJECT_NAME} test_package.cpp)
target_link_libraries(${PROJECT_NAME} hello::libhello)
        """)
    t = TestClient()
    t.save({'conanfile.py': conanfile_py,
            'hello.cpp': hello_cpp,
            'hello.h': hello_h,
            'CMakeLists.txt': cmakelists_txt,
            'test_package/conanfile.py': test_conanfile_py,
            'test_package/CMakeLists.txt': test_cmakelists_txt,
            'test_package/test_package.cpp': test_test_package_cpp})
    t.run("create . --name=hello --version=1.0")


@pytest.mark.tool("cmake")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
def test_iphoneos_crossbuild():
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os=iOS
        os.version=12.0
        os.sdk=iphoneos
        arch=armv8
    """).format()

    client = TestClient(path_with_spaces=False)
    client.save({"ios-armv8": profile}, clean_first=True)
    client.run("new cmake_lib -d name=hello -d version=0.1")
    client.run("create . --profile:build=default --profile:host=ios-armv8 -tf=\"\"")

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    # FIXME: The crossbuild for iOS etc is failing with find_package because cmake ignore the
    #        cmake_prefix_path to point only to the Frameworks of the system. The only fix found
    #        would require to introduce something like "set (mylibrary_DIR "${CMAKE_BINARY_DIR}")"
    #        at the toolchain (but it would require the toolchain to know about the deps)
    #        https://stackoverflow.com/questions/65494246/cmakes-find-package-ignores-the-paths-option-when-building-for-ios#
    cmakelists = textwrap.dedent("""
    set(CMAKE_CXX_COMPILER_WORKS 1)
    set(CMAKE_CXX_ABI_COMPILED 1)
    cmake_minimum_required(VERSION 3.15)
    project(MyApp CXX)
    set(hello_DIR "${CMAKE_BINARY_DIR}")
    find_package(hello)
    add_executable(main main.cpp)
    target_link_libraries(main hello::hello)
    """)

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake

        class TestConan(ConanFile):
            requires = "hello/0.1"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "CMakeLists.txt", "main.cpp"
            generators = "CMakeDeps", "CMakeToolchain"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)

    client.save({"conanfile.py": conanfile,
                 "CMakeLists.txt": cmakelists,
                 "main.cpp": main,
                 "ios-armv8": profile}, clean_first=True)
    client.run("install . --profile:build=default --profile:host=ios-armv8")
    client.run("build . --profile:build=default --profile:host=ios-armv8")
    main_path = "./main.app/main"
    client.run_command("lipo -info {}".format(main_path))
    assert "Non-fat file" in client.out
    assert "is architecture: arm64" in client.out
    client.run_command(f"vtool -show-build {main_path}")
    assert "platform IOS" in client.out
    assert "minos 12.0" in client.out



import os
import textwrap

from conan.test.utils.tools import TestClient
from conan.internal.util.files import load, mkdir

conanfile = '''
from conan import ConanFile
from conan.tools.files import save, load
from conan.tools.files import copy
import os

class ConanFileToolsTest(ConanFile):
    name = "pkg"
    version = "0.1"
    exports_sources = "*"

    def layout(self):
        self.folders.build = "../build"
        self.folders.source = "."

    def build(self):
        self.output.info("Source files: %s" % load(self, os.path.join(self.source_folder, "file.h")))
        save(self, "myartifact.lib", "artifact contents!")
        save(self, "subdir/myartifact2.lib", "artifact2 contents!")

    def package(self):
        copy(self, "*.h", self.source_folder, self.package_folder)
        copy(self, "*.lib", self.build_folder, self.package_folder)
'''


class TestDevInSourceFlow:

    @staticmethod
    def _assert_pkg(folder):
        assert sorted(['file.h', 'myartifact.lib', 'subdir', 'conaninfo.txt',
                       'conanmanifest.txt']) == sorted(os.listdir(folder))
        assert load(os.path.join(folder, "myartifact.lib")) == "artifact contents!"
        assert load(os.path.join(folder, "subdir/myartifact2.lib")) == "artifact2 contents!"

    def test_parallel_folders(self):
        client = TestClient(light=True)
        repo_folder = os.path.join(client.current_folder, "recipe")
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(repo_folder)
        mkdir(build_folder)

        client.current_folder = repo_folder  # equivalent to git clone recipe
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.current_folder = build_folder
        client.run("install ../recipe")
        client.run("build ../recipe")
        client.current_folder = repo_folder
        client.run("export . --user=lasote --channel=testing")
        client.run("export-pkg . --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)

    def test_insource_build(self):
        client = TestClient(light=True)
        repo_folder = client.current_folder
        package_folder = os.path.join(client.current_folder, "pkg")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.run("install .")
        client.run("build .")
        client.current_folder = repo_folder
        client.run("export . --user=lasote --channel=testing")
        client.run("export-pkg . --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)

    def test_child_build(self):
        client = TestClient(light=True)
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        package_folder = os.path.join(build_folder, "package")
        mkdir(package_folder)
        client.save({"conanfile.py": conanfile,
                     "file.h": "file_h_contents!"})

        client.current_folder = build_folder
        client.run("install ..")
        client.run("build ..")
        client.run("export-pkg .. --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)


conanfile_out = '''
from conan import ConanFile
from conan.tools.files import save, load
from conan.tools.files import copy
import os

class ConanFileToolsTest(ConanFile):
    name = "pkg"
    version = "0.1"

    def source(self):
        save(self, os.path.join(self.source_folder, "file.h"), "file_h_contents!")

    def build(self):
        self.output.info("Source files: %s" % load(self, os.path.join(self.source_folder, "file.h")))
        save(self, "myartifact.lib", "artifact contents!")

    def package(self):
        copy(self, "*.h", self.source_folder, self.package_folder)
        copy(self, "*.lib", self.build_folder, self.package_folder)
'''


class TestDevOutSourceFlow:

    @staticmethod
    def _assert_pkg(folder):
        assert sorted(['file.h', 'myartifact.lib', 'conaninfo.txt', 'conanmanifest.txt']) == \
                         sorted(os.listdir(folder))

    def test_parallel_folders(self):
        client = TestClient(light=True)
        repo_folder = os.path.join(client.current_folder, "recipe")
        src_folder = os.path.join(client.current_folder, "src")
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(repo_folder)
        mkdir(src_folder)
        mkdir(build_folder)
        client.current_folder = repo_folder  # equivalent to git clone recipe
        conanfile_final = conanfile_out + """
    def layout(self):
        self.folders.build = "../build"
        self.folders.source = "../src"
        """
        client.save({"conanfile.py": conanfile_final})

        client.current_folder = build_folder
        client.run("install ../recipe")
        client.current_folder = src_folder  # FIXME: Source layout not working
        client.run("source ../recipe")

        client.current_folder = build_folder
        client.run("build ../recipe")
        client.current_folder = repo_folder
        client.run("export . --user=lasote --channel=testing")
        client.run("export-pkg . --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)

    def test_insource_build(self):
        client = TestClient(light=True)
        repo_folder = client.current_folder
        client.save({"conanfile.py": conanfile_out})

        client.run("install .")
        client.run("source .")
        client.run("build . ")

        client.current_folder = repo_folder
        client.run("export . --user=lasote --channel=testing")
        client.run("export-pkg . --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)

    def test_child_build(self):
        client = TestClient(light=True)
        repo_folder = client.current_folder
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        conanfile_final = conanfile_out + """
    def layout(self):
        self.folders.build = "build"
        """
        client.save({"conanfile.py": conanfile_final})

        client.current_folder = build_folder
        client.run("install ..")
        client.current_folder = repo_folder  # FIXME: Source layout not working
        client.run("source .")
        client.current_folder = build_folder
        client.run("build ..")
        client.current_folder = repo_folder

        client.run("export-pkg . --name=pkg --version=0.1 --user=lasote --channel=testing")

        cache_package_folder = client.created_layout().package()
        self._assert_pkg(cache_package_folder)


def test_error_build_file():
    c = TestClient(light=True)
    conanfile_error = textwrap.dedent(r"""
        from conan import ConanFile
        class Test(ConanFile):
            def layout(self):
                self.folders.build = "build"
        """)

    c.save({"conanfile.py": conanfile_error,
            "build": "# Some existing BUILD file like bazel"})
    c.run("build . ", assert_error=True)
    assert "ERROR: conanfile.py: Failed to create build folder, there is already a file" in c.out



import os
import textwrap
import json
import pytest

from conan.internal.paths import CONANFILE
from conan.test.utils.tools import TestClient
from conan.internal.cache.conan_reference_layout import EXPORT_FOLDER

conanfile_py = """
from conan import ConanFile

class AConan(ConanFile):
    name = "package"
    version = "0.1.0"
"""

with_deps_path_file = """
from conan import ConanFile

class BConan(ConanFile):
    name = "package2"
    version = "0.2.0"
    requires = "package/0.1.0@user/testing"
"""

deps_txt_file = """
[requires]
package2/0.2.0@user/testing
"""


@pytest.fixture()
def client_deps():
    client = TestClient()
    client.save({CONANFILE: conanfile_py})
    client.run(f"export . --user=user --channel=testing")
    client.save({CONANFILE: with_deps_path_file}, clean_first=True)
    client.run(f"export . --user=user --channel=testing")
    client.save({'conanfile.txt': deps_txt_file}, clean_first=True)
    return client


def test_basic():
    client = TestClient()
    client.save({CONANFILE: conanfile_py})
    client.run(f"export . --user=user --channel=testing")
    client.run(f"graph info --requires=package/0.1.0@user/testing --format=json")
    nodes = json.loads(client.stdout)["graph"]["nodes"]
    assert client.cache_folder in nodes["1"]["recipe_folder"]
    assert os.path.basename(nodes["1"]["recipe_folder"]).strip() == EXPORT_FOLDER
    assert nodes["1"]["source_folder"] is None
    assert nodes["1"]["build_folder"] is None
    assert nodes["1"]["package_folder"] is None


def test_build_id():
    # https://github.com/conan-io/conan/issues/6915
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            options = {"myOption": [True, False]}
            def build_id(self):
                self.info_build.options.myOption = "Any"
        """)
    client.save({CONANFILE: conanfile})
    client.run(f"export . --name=pkg --version=0.1 --user=user --channel=testing")
    client.run(f"graph info --requires=pkg/0.1@user/testing -o pkg/*:myOption=True")
    out = str(client.out).replace("\\", "/")
    assert "package_id: b868c8ab4ae6ddccfe19fabd62a5e180d4b18a2b" in out
    assert "build_id: d5d6fc54af6f589e338090910ac18c848a87720d" in out

    client.run("graph info --requires=pkg/0.1@user/testing -o pkg/*:myOption=False")
    out = str(client.out).replace("\\", "/")
    assert "package_id: 41e2e23ac9570fd23f421bcd0cf9e5cbab49e6ee" in out
    assert "build_id: d5d6fc54af6f589e338090910ac18c848a87720d" in out


def test_deps_basic(client_deps):
    for ref in [f"--requires=package2/0.2.0@user/testing", "conanfile.txt"]:
        client_deps.run(f"graph info {ref} --format=json")
        nodes = json.loads(client_deps.stdout)
        found_ref = False
        assert len(nodes["graph"]["nodes"]) == 3

        for _, node in nodes["graph"]["nodes"].items():
            if node["ref"] == "conanfile":
                assert node["source_folder"] is None
            else:
                assert client_deps.cache_folder in node["recipe_folder"]
                assert os.path.basename(node["recipe_folder"]).strip() == EXPORT_FOLDER
            assert node["source_folder"] is None
            assert node["build_folder"] is None
            assert node["package_folder"] is None
            found_ref = found_ref or "package/0.1.0@user/testing" in node["ref"]
        assert found_ref


def test_deps_specific_information(client_deps):
    client_deps.run("graph info . --package-filter package/* --format=json")
    nodes = json.loads(client_deps.stdout)["graph"]["nodes"]
    assert len(nodes) == 1
    assert "package/0.1.0@user/testing" in nodes["2"]["ref"]
    assert nodes["2"]["source_folder"] is None
    assert nodes["2"]["build_folder"] is None
    assert nodes["2"]["package_folder"] is None

    client_deps.run("graph info . --package-filter package* --format=json")
    nodes = json.loads(client_deps.stdout)["graph"]["nodes"]
    assert len(nodes) == 2
    assert "package2/0.2.0@user/testing" in nodes["1"]["ref"]
    assert nodes["1"]["source_folder"] is None
    assert nodes["1"]["build_folder"] is None
    assert nodes["1"]["package_folder"] is None
    assert "package/0.1.0@user/testing" in nodes["2"]["ref"]
    assert nodes["2"]["source_folder"] is None
    assert nodes["2"]["build_folder"] is None
    assert nodes["2"]["package_folder"] is None


def test_single_field():
    client = TestClient()
    client.save({CONANFILE: conanfile_py})
    client.run(f"export . --user=user --channel=testing")
    client.run(f"graph info --requires package/0.1.0@user/testing --format=json")
    nodes = json.loads(client.stdout)["graph"]["nodes"]
    assert len(nodes) == 2
    assert "package/0.1.0@user/testing" in nodes["1"]["ref"]
    assert nodes["1"]["source_folder"] is None
    assert nodes["1"]["build_folder"] is None
    assert nodes["1"]["package_folder"] is None


def test_direct_conanfile():
    client = TestClient()
    client.save({CONANFILE: conanfile_py})
    client.run("graph info .")
    output = client.out
    assert "export_folder" not in output
    assert "source_folder: None" in output
    assert "build_folder: None" in output
    assert "package_folder: None" in output



import pytest
from conan.tools.microsoft import msvs_toolset
from conan.errors import ConanException
from conan.test.utils.mocks import ConanFileMock, MockSettings


def test_invalid_compiler():
    """ When the compiler version is unknown and there is no toolset configured,
        msvs_toolset must return None
    """
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "Visual Studio",
                                       "compiler.version": "20220203"})
    assert msvs_toolset(conanfile) is None


@pytest.mark.parametrize("compiler_version,expected_toolset", [
    ("195", "v145"),
    ("194", "v143"),
    ("193", "v143"),
    ("192", "v142"),
    ("191", "v141"),
    ("190", "v140"),
    ("180", "v120"),
    ("170", "v110")])
def test_msvc_default(compiler_version, expected_toolset):
    """ When running msvc as compiler, and there is no toolset configured,
        msvs_toolset must return a specific version based on the compiler version
    """
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "msvc", "compiler.version": compiler_version})
    assert msvs_toolset(conanfile) == expected_toolset


@pytest.mark.parametrize("compiler_version,toolset,expected_toolset", [
    ("193", "v143_xp", "v143_xp"),
    ("192", "v142_xp", "v142_xp"),
    ("191", "v141_xp", "v141_xp"),
    ("190", "v140_xp", "v140_xp"),
    ("180", "v120_xp", "v120_xp"),
    ("170", "v110_xp", "v110_xp")])
def test_msvc_custom(compiler_version, toolset, expected_toolset):
    """ When running msvc as compiler, and there is a toolset configured,
        msvs_toolset must return the specified toolset from profile
    """
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "msvc",
                             "compiler.version": compiler_version,
                             "compiler.toolset": toolset})
    assert msvs_toolset(conanfile) == expected_toolset


def test_intel_cc_old_compiler():
    """ When running intel-cc as compiler, and the compiler version configured is older than 2021
        msvs_toolset must raise an ConanExpection
    """
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "intel-cc", "compiler.version": "19.1"})
    with pytest.raises(ConanException) as error:
        msvs_toolset(conanfile)
    assert "You have to use 'intel' compiler" in str(error.value)


@pytest.mark.parametrize("compiler_version,compiler_mode,expected_toolset", [
    ("2021.3", "classic", "Intel C++ Compiler 19.2"),
    ("2021.2", "classic", "Intel C++ Compiler 19.2"),
    ("2021.1", "classic", "Intel C++ Compiler 19.2"),
    ("2021.3", "icx", "Intel C++ Compiler 2021"),
    ("2021.2", "icx", "Intel C++ Compiler 2021"),
    ("2021.1", "icx", "Intel C++ Compiler 2021"),
    ("2021.3", "dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("2021.2", "dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("2021.1", "dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("2021.3", None, "Intel(R) oneAPI DPC++ Compiler")])
def test_intel_cc_default(compiler_version, compiler_mode, expected_toolset):
    """ When running intel-cc as compiler, and there is a proper compiler version configured,
        and the compiler.mode is configured,
        msvs_toolset must return a compiler toolset based on compiler.mode
    """
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "intel-cc",
                                       "compiler.version": compiler_version,
                                       "compiler.mode": compiler_mode})
    assert msvs_toolset(conanfile) == expected_toolset

def test_clang_platform_toolset():
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": "clang",
                             "compiler.version": "17",
                             "compiler.runtime": "dynamic"})
    assert msvs_toolset(conanfile) == "ClangCl"



import json
import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load


def test_transitive_py_requires():
    # https://github.com/conan-io/conan/issues/5529
    client = TestClient(light=True)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class PackageInfo(ConanFile):
            python_requires = "dep/[>0.0]@user/channel"
        """)
    consumer = textwrap.dedent("""
        from conan import ConanFile
        class MyConanfileBase(ConanFile):
            python_requires = "pkg/0.1@user/channel"
        """)
    client.save({"dep/conanfile.py": GenConanfile(),
                 "pkg/conanfile.py": conanfile,
                 "consumer/conanfile.py": consumer})

    client.run("export dep --name=dep --version=0.1 --user=user --channel=channel")
    client.run("export pkg --name=pkg --version=0.1 --user=user --channel=channel")
    client.run("lock create consumer/conanfile.py")

    client.run("export dep --name=dep --version=0.2 --user=user --channel=channel")

    client.run("install consumer/conanfile.py")
    assert "dep/0.1@user/channel" in client.out
    assert "dep/0.2" not in client.out

    os.remove(os.path.join(client.current_folder, "consumer/conan.lock"))
    client.run("install consumer/conanfile.py")
    assert "dep/0.2@user/channel" in client.out
    assert "dep/0.1" not in client.out


def test_transitive_matching_ranges():
    client = TestClient(light=True)
    tool = textwrap.dedent("""
        from conan import ConanFile
        class PackageInfo(ConanFile):
            python_requires = "dep/{}"
        """)
    pkg = textwrap.dedent("""
        from conan import ConanFile
        class MyConanfileBase(ConanFile):
            python_requires = "tool/{}"
            def configure(self):
                for k, p in self.python_requires.items():
                    self.output.info("%s: %s!!" % (k, p.ref))
        """)
    client.save({"dep/conanfile.py": GenConanfile(),
                 "tool1/conanfile.py": tool.format("[<0.2]"),
                 "tool2/conanfile.py": tool.format("[>0.0]"),
                 "pkga/conanfile.py": pkg.format("[<0.2]"),
                 "pkgb/conanfile.py": pkg.format("[>0.0]"),
                 "app/conanfile.py": GenConanfile().with_requires("pkga/[*]", "pkgb/[*]")})

    client.run("export dep --name=dep --version=0.1")
    client.run("export dep --name=dep --version=0.2")
    client.run("export tool1 --name=tool --version=0.1")
    client.run("export tool2 --name=tool --version=0.2")
    client.run("create pkga --name=pkga --version=0.1")
    client.run("create pkgb --name=pkgb --version=0.1")
    client.run("lock create app/conanfile.py --lockfile-out=app.lock")

    client.run("export dep --name=dep --version=0.2")
    client.run("export tool2 --name=tool --version=0.3")
    client.run("create pkga --name=pkga --version=0.2")
    client.run("create pkgb --name=pkgb --version=0.2")

    client.run("install app/conanfile.py --lockfile=app.lock")
    assert "pkga/0.1: tool: tool/0.1!!" in client.out
    assert "pkga/0.1: dep: dep/0.1!!" in client.out
    assert "pkgb/0.1: tool: tool/0.2!!" in client.out
    assert "pkgb/0.1: dep: dep/0.2!!" in client.out

    client.run("install app/conanfile.py")
    assert "pkga/0.2: tool: tool/0.1!!" in client.out
    assert "pkga/0.2: dep: dep/0.1!!" in client.out
    assert "pkgb/0.2: tool: tool/0.3!!" in client.out
    assert "pkgb/0.2: dep: dep/0.2!!" in client.out


def test_lock_pyrequires_prereleases():
    tc = TestClient(light=True)
    tc.save({"dep/conanfile.py": GenConanfile("dep", "1.0-0.1"),
             "app/conanfile.py": GenConanfile("app", "1.0").with_python_requires("dep/[>=0]")})
    tc.run("export dep")
    dep_rrev = tc.exported_recipe_revision()
    tc.run("lock create app --lockfile-out=app.lock", assert_error=True)
    # Makes sense, prereleases are not active
    assert ("Version range '>=0' from requirement 'dep/[>=0]' required by "
            "'python_requires' could not be resolved") in tc.out

    tc.save_home({"global.conf": "core.version_ranges:resolve_prereleases=True"})
    # This used to crash even with the conf activated
    tc.run("lock create app --lockfile-out=app.lock")
    data = json.loads(load(os.path.join(tc.current_folder, "app.lock")))
    assert data["python_requires"][0].startswith(f"dep/1.0-0.1#{dep_rrev}")


def test_lock_pyrequires_create():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("tool", "0.1").with_package_type("python-require")})
    c.run("create .  --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#7835890f1e86b3f7fc6d76b4e3c43cb1" in lock["python_requires"][0]


def test_lock_pyrequires_export():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("tool", "0.1").with_package_type("python-require")})
    c.run("export .  --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#7835890f1e86b3f7fc6d76b4e3c43cb1" in lock["python_requires"][0]


def test_lock_pyrequires_export_transitive():
    c = TestClient(light=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_package_type("python-require"),
            "tool/conanfile.py": GenConanfile("tool", "0.1").with_package_type("python-require")
                                                            .with_python_requires("dep/0.1")})
    c.run("export dep")
    c.run("export tool --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#c0008d3fcf07f7a690dd16bf6c171cec" in lock["python_requires"][0]
    assert "dep/0.1#5d31586a2a4355d68898875dc591009a" in lock["python_requires"][1]


def test_lock_export_transitive_pyrequire():
    c = TestClient(light=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_package_type("python-require"),
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_python_requires("dep/0.1")})
    c.run("export dep")
    c.run("export pkg --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "pkg/0.1#dfd6bc1becb3915043a671111860baee" in lock["requires"][0]
    assert "dep/0.1#5d31586a2a4355d68898875dc591009a" in lock["python_requires"][0]


def test_pyrequires_test_package_lockfile_error():
    # https://github.com/conan-io/conan/issues/19340
    c = TestClient(light=True)
    c.save({"utils/conanfile.py": GenConanfile("utils").with_package_type("python-require"),
            "bar/conanfile.py": GenConanfile("bar", "8.0").with_python_requires("utils/[^1]"),
            "bar/test_package/conanfile.py": GenConanfile().with_test("pass")
                                                           .with_python_requires("utils/[^1]"),
            "foo/conanfile.py": GenConanfile("foo", "1.0").with_requires("bar/[*]")
                                                          .with_python_requires("utils/1.7.1")})

    c.run("create utils --version=1.7.1")
    c.run("create utils --version=1.8.0")
    c.run("create bar --lockfile-out=bar.lock")
    c.run("create foo", assert_error=True)
    assert "Missing prebuilt package for 'bar/8.0'" in c.out

    c.run("create foo --lockfile=bar.lock --lockfile-partial --lockfile-out=foo.lock")
    c.assert_listed_require({"utils/1.7.1": "Cache",
                             "utils/1.8.0": "Cache"}, python=True)
    lock = json.loads(c.load("foo.lock"))
    assert "utils/1.8.0" in lock["python_requires"][0]
    assert "utils/1.7.1" in lock["python_requires"][1]

    # The lockfile works, because the fixed version is an older one, and the lock can have
    # both, the range resolves to 1.8.0, and the fixed resolves to 1.7.1
    # This lockfile can be used with same results
    c.run("create foo --lockfile=foo.lock")
    c.assert_listed_require({"utils/1.7.1": "Cache",
                             "utils/1.8.0": "Cache"}, python=True)


def test_pyrequires_test_package_lockfile_error_forward():
    # https://github.com/conan-io/conan/issues/19340
    c = TestClient(light=True)
    c.save({"utils/conanfile.py": GenConanfile("utils").with_package_type("python-require"),
            "bar/conanfile.py": GenConanfile("bar", "8.0").with_python_requires("utils/[^1]"),
            "bar/test_package/conanfile.py": GenConanfile().with_test("pass")
                                                           .with_python_requires("utils/[^1]"),
            "foo/conanfile.py": GenConanfile("foo", "1.0").with_requires("bar/[*]")
                                                          .with_python_requires("utils/1.9.0")})

    # bar binary is built with utils/1.8.0
    c.run("create utils --version=1.8.0")
    c.run("create bar --lockfile-out=bar.lock")
    c.assert_listed_require({"utils/1.8.0": "Cache"}, python=True)
    lock = json.loads(c.load("bar.lock"))
    assert "utils/1.8.0" in lock["python_requires"][0]

    # now a new utils/1.9.0 is out, so the regular consumption of bar will fail with missing binary
    # this will happen even if foo doesn't declare python-requires
    c.run("create utils --version=1.9.0")
    c.run("create foo", assert_error=True)
    assert "Missing prebuilt package for 'bar/8.0'" in c.out
    c.assert_listed_require({"utils/1.9.0": "Cache"}, python=True)

    # If we try to force bar strict dependencies with lockfile, it will fail, as expected
    c.run("create foo --lockfile=bar.lock", assert_error=True)
    assert "Requirement 'utils/1.9.0' not in lockfile 'python_requires'" in c.out

    # we can relax the lockfile, and it will be able to resolve,
    # but it will still fail with missing binary for bar
    c.run("create foo --lockfile=bar.lock --lockfile-partial", assert_error=True)
    assert "Missing prebuilt package for 'bar/8.0'" in c.out

    c.run("create foo --lockfile=bar.lock --lockfile-partial --build=missing "
          "--lockfile-out=foo.lock")
    c.assert_listed_require({"utils/1.9.0": "Cache"}, python=True)
    assert "utils/1.8.0" not in c.out

    # This build is reproducible with this lockfile
    c.run("create foo --lockfile=foo.lock")
    c.assert_listed_require({"utils/1.9.0": "Cache"}, python=True)
    assert "utils/1.8.0" not in c.out



import os

from conans.server.utils.files import path_exists
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import mkdir


def test_path_exists():
    """
    Unit test of path_exists
    """
    tmp_dir = temp_folder()
    tmp_dir = os.path.join(tmp_dir, "WhatEver")
    new_path = os.path.join(tmp_dir, "CapsDir")
    mkdir(new_path)
    assert path_exists(new_path, tmp_dir)
    assert not path_exists(os.path.join(tmp_dir, "capsdir"), tmp_dir)



LOCAL_RECIPES_INDEX = "local-recipes-index"


class Remote:
    """
    The ``Remote`` class represents a remote registry of packages.
    """
    def __init__(self, name, url, verify_ssl=True, disabled=False, allowed_packages=None,
                 remote_type=None, recipes_only=False):
        """ A Remote object can be constructed to be passed as an argument to
        RemotesAPI methods. When possible, it is better to use Remote objects returned by the API,
        but for the ``RemotesAPI.add()`` method, for which a new constructed object is necessary.
        It is recommended to use named arguments like ``Remote(..., verify_ssl=False)`` in
        the constructor.
        :param name: The name of the remote.
        :param url: The URL of the remote repository (or local folder for "local-recipes-index").
        :param verify_ssl: Enable SSL Certificate validation.
        :param disabled: Disable the remote repository.
        :param allowed_packages: List of patterns of allowed packages from this remote
        :param remote_type: Type of the remote repository, use "local-recipes-index" or ``None``
        :param recipes_only: If True, binaries form this remote will be ignored and never used
        """
        self.name = name  # Read only, is the key
        self.url = url
        self.verify_ssl = verify_ssl
        self.disabled = disabled
        self.allowed_packages = allowed_packages
        self.recipes_only = recipes_only
        self.remote_type = remote_type
        self._caching = {}

    def __eq__(self, other):
        if other is None:
            return False
        return (self.name == other.name and self.url == other.url and
                self.verify_ssl == other.verify_ssl and self.disabled == other.disabled)

    def __str__(self):
        allowed_msg = ""
        if self.allowed_packages:
            allowed_msg = ", Allowed packages: {}".format(", ".join(self.allowed_packages))
        if self.recipes_only:
            allowed_msg += ", Recipes only"
        if self.remote_type == LOCAL_RECIPES_INDEX:
            return "{}: {} [{}, Enabled: {}{}]".format(self.name, self.url, LOCAL_RECIPES_INDEX,
                                                       not self.disabled, allowed_msg)
        return "{}: {} [Verify SSL: {}, Enabled: {}{}]".format(self.name, self.url, self.verify_ssl,
                                                               not self.disabled, allowed_msg)

    def __repr__(self):
        return str(self)

    def invalidate_cache(self):
        """ If external operations might have modified the remote since it was instantiated,
        this method can be called to invalidate the cache.
        Note that this is done automatically when the remote is used in any operation by Conan,
        such as uploading packages, so this method is not usually needed when only interacting
        with the Conan API"""
        self._caching = {}



import platform
import re
import textwrap

import pytest

from conan.test.utils.tools import TestClient


def _get_filename(configuration, architecture, sdk_version):
    props = [("configuration", configuration),
             ("architecture", architecture),
             ("sdk version", sdk_version)]
    name = "".join("_{}".format(v) for _, v in props if v is not None and v)
    name = name.replace(".", "_").replace("-", "_")
    return name.lower()


def _condition(configuration, architecture, sdk_version):
    sdk = "macosx{}".format(sdk_version or "*")
    return "[config={}][arch={}][sdk={}]".format(configuration, architecture, sdk)


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.parametrize("configuration, os_version, libcxx, cppstd, arch, sdk_version, clang_cppstd", [
    ("Release", "", "", "", "x86_64", "", ""),
    ("Debug", "", "", "", "armv8", "", ""),
    ("Release", "12.0", "libc++", "20", "x86_64", "", "c++20"),
    ("Debug", "12.0", "libc++", "20", "x86_64", "", "c++20"),
    ("Release", "12.0", "libc++", "20", "x86_64", "11.3", "c++20"),
    ("Release", "12.0", "libc++", "20", "x86_64", "", "c++20"),
])
def test_toolchain_files(configuration, os_version, cppstd, libcxx, arch, sdk_version, clang_cppstd):
    client = TestClient()
    client.save({"conanfile.txt": "[generators]\nXcodeToolchain\n"})
    cmd = "install . -s build_type={}".format(configuration)
    cmd = cmd + " -s os.version={}".format(os_version) if os_version else cmd
    cmd = cmd + " -s compiler.cppstd={}".format(cppstd) if cppstd else cmd
    cmd = cmd + " -s os.sdk_version={}".format(sdk_version) if sdk_version else cmd
    cmd = cmd + " -s arch={}".format(arch) if arch else cmd
    client.run(cmd)
    arch_name = "arm64" if arch == "armv8" else arch
    filename = _get_filename(configuration, arch_name, sdk_version)
    condition = _condition(configuration, arch, sdk_version)

    toolchain_all = client.load("conantoolchain.xcconfig")
    toolchain_vars = client.load("conantoolchain{}.xcconfig".format(filename))
    conan_config = client.load("conan_config.xcconfig")

    assert '#include "conantoolchain.xcconfig"' in conan_config
    assert '#include "conantoolchain{}.xcconfig"'.format(filename) in toolchain_all

    if libcxx:
        assert 'CLANG_CXX_LIBRARY{}={}'.format(condition, libcxx) in toolchain_vars
    if os_version:
        assert 'MACOSX_DEPLOYMENT_TARGET{}={}'.format(condition, os_version) in toolchain_vars
    if cppstd:
        assert 'CLANG_CXX_LANGUAGE_STANDARD{}={}'.format(condition, clang_cppstd) in toolchain_vars


def test_toolchain_flags():
    client = TestClient()
    client.save({"conanfile.txt": "[generators]\nXcodeToolchain\n"})
    cmd = "install . -c 'tools.build:cxxflags=[\"flag1\"]' " \
          "-c 'tools.build:defines=[\"MYDEFINITION\"]' " \
          "-c 'tools.build:cflags=[\"flag2\"]' " \
          "-c 'tools.build:sharedlinkflags=[\"flag3\"]' " \
          "-c 'tools.build:exelinkflags=[\"flag4\"]'"
    client.run(cmd)
    conan_global_flags = client.load("conan_global_flags.xcconfig")
    assert "GCC_PREPROCESSOR_DEFINITIONS = $(inherited) MYDEFINITION" in conan_global_flags
    assert "OTHER_CFLAGS = $(inherited) flag2" in conan_global_flags
    assert "OTHER_CPLUSPLUSFLAGS = $(inherited) flag1" in conan_global_flags
    assert "OTHER_LDFLAGS = $(inherited) flag3 flag4" in conan_global_flags
    conan_global_file = client.load("conan_config.xcconfig")
    assert '#include "conan_global_flags.xcconfig"' in conan_global_file


def test_flags_generated_if_only_defines():
    # https://github.com/conan-io/conan/issues/16422
    client = TestClient()
    client.save({"conanfile.txt": "[generators]\nXcodeToolchain\n"})
    client.run("install . -c 'tools.build:defines=[\"MYDEFINITION\"]'")
    conan_global_flags = client.load("conan_global_flags.xcconfig")
    assert "GCC_PREPROCESSOR_DEFINITIONS = $(inherited) MYDEFINITION" in conan_global_flags
    conan_global_file = client.load("conan_config.xcconfig")
    assert '#include "conan_global_flags.xcconfig"' in conan_global_file


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.parametrize("os_name, sdk, min_version, deployment_target_flag", [
    ("Macos", None, "11.0", "MACOSX_DEPLOYMENT_TARGET"),
    ("iOS", "iphoneos", "18.0", "IPHONEOS_DEPLOYMENT_TARGET"),
    ("tvOS", "appletvos", "18.4", "TVOS_DEPLOYMENT_TARGET"),
    ("watchOS", "watchos", "9.0", "WATCHOS_DEPLOYMENT_TARGET"),
    ("visionOS", "xros", "2.0", "XROS_DEPLOYMENT_TARGET"),
])
def test_xcodetoolchain_xcconfig_deplyment_target(os_name, sdk, min_version, deployment_target_flag):
    client = TestClient()

    conanfile = textwrap.dedent(f"""
        import os
        from conan import ConanFile
        from conan.tools.apple import XcodeToolchain
        from conan.tools.files import save
        class MyApplicationConan(ConanFile):
            name = "myapplication"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def generate(self):
                tc = XcodeToolchain(self)
                tc.generate()
                # Private access to get the generated xcconfig filename
                # It changes based on the settings, so this is the less fragile way to get it
                save(self, os.path.join(self.generators_folder, "name.txt"), tc._vars_xconfig_filename)
        """)

    client.save({"conanfile.py": conanfile})

    settings = f"-s os={os_name} -s os.version={min_version}"
    if sdk:
        settings += f" -s os.sdk={sdk}"

    client.run(f"install . -s build_type=Release {settings} --build=missing")

    xcconfig_name = client.load("name.txt").strip()
    xcconfig = client.load(xcconfig_name)
    match = re.search(f"^{deployment_target_flag}.+={min_version}$", xcconfig, re.MULTILINE)
    assert match is not None



import os
import platform

import pytest

from conan.test.utils.mocks import ConanFileMock
from conan.tools.env.environment import environment_wrap_command
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient


def editable_cmake(generator, build_folder=None):
    c = TestClient()
    if generator is not None:
        c.save_home({"global.conf": "tools.cmake.cmaketoolchain:generator={}".format(generator)})
    with c.chdir("dep"):
        c.run("new cmake_lib -d name=dep -d version=0.1")
    with c.chdir("pkg"):
        c.run("new cmake_exe -d name=pkg -d version=0.1 -d requires=dep/0.1")

    output_folder = '--output-folder="{}"'.format(build_folder) if build_folder else ""

    def build_dep():
        c.run("build . {}".format(output_folder))
        c.run("build . -s build_type=Debug {}".format(output_folder))

    with c.chdir("dep"):
        c.run("editable add . {}".format(output_folder))
        build_dep()

    def build_pkg(msg, build_editable=False):
        build_editable = "--build=editable" if build_editable else ""
        c.run(f"build . {build_editable}")
        folder = os.path.join("build", "Release")
        c.run_command(os.sep.join([".", folder, "pkg"]))
        assert "pkg/0.1: Hello World Release!" in c.out
        assert "{}: Hello World Release!".format(msg) in c.out
        c.run(f"build . -s build_type=Debug {build_editable}")
        folder = os.path.join("build", "Debug")
        c.run_command(os.sep.join([".", folder, "pkg"]))
        assert "pkg/0.1: Hello World Debug!" in c.out
        assert "{}: Hello World Debug!".format(msg) in c.out

    with c.chdir("pkg"):
        c.run("install . ")
        c.run("install . -s build_type=Debug")
        build_pkg("dep/0.1")

    # Do a source change in the editable!
    with c.chdir("dep"):
        f = c.load("src/dep.cpp")
        f = f.replace("dep/0.1", "SUPERDEP/0.1")
        c.save({"src/dep.cpp": f})
        build_dep()

    with c.chdir("pkg"):
        build_pkg("SUPERDEP/0.1")

    # Do a source change in the editable, but do NOT build yet
    with c.chdir("dep"):
        f = c.load("src/dep.cpp")
        f = f.replace("SUPERDEP/0.1", "MEGADEP/0.1")
        c.save({"src/dep.cpp": f})
    # And build with --build=editable
    with c.chdir("pkg"):
        build_pkg("MEGADEP/0.1", build_editable=True)

    # Check that create is still possible
    c.run("editable remove dep")
    c.run("create dep")
    c.run("create pkg")
    # print(c.out)
    assert "pkg/0.1: Created package" in c.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.parametrize("generator", [None, "MinGW Makefiles"])
@pytest.mark.tool("mingw64")
def test_editable_cmake_windows(generator):
    editable_cmake(generator)


@pytest.mark.tool("cmake")
def test_editable_cmake_windows_folders():
    build_folder = temp_folder()
    editable_cmake(generator=None, build_folder=build_folder)


@pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
@pytest.mark.parametrize("generator", [None, "Ninja", "Ninja Multi-Config"])
@pytest.mark.tool("cmake", "3.19")
def test_editable_cmake_linux(generator):
    editable_cmake(generator)


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Macos")
@pytest.mark.parametrize("generator", [None, "Ninja", "Xcode"])
@pytest.mark.tool("cmake", "3.23")
def test_editable_cmake_osx(generator):
    editable_cmake(generator)


def editable_cmake_exe(generator):
    c = TestClient()
    if generator is not None:
        c.save_home({"global.conf": "tools.cmake.cmaketoolchain:generator={}".format(generator)})

    c.run("new cmake_exe -d name=dep -d version=0.1 -o=dep")

    def build_dep():
        c.run("build .")
        c.run("build . -s build_type=Debug")

    with c.chdir("dep"):
        c.run("editable add .")
        build_dep()

    def run_pkg(msg):
        host_arch = c.get_default_host_profile().settings['arch']
        cmd_release = environment_wrap_command(ConanFileMock(), f"conanrunenv-release-{host_arch}",
                                               c.current_folder, "dep")
        c.run_command(cmd_release)
        assert "{}: Hello World Release!".format(msg) in c.out
        cmd_release = environment_wrap_command(ConanFileMock(), f"conanrunenv-debug-{host_arch}",
                                               c.current_folder, "dep")
        c.run_command(cmd_release)
        assert "{}: Hello World Debug!".format(msg) in c.out

    with c.chdir("pkg"):
        c.run("install --requires=dep/0.1")
        c.run("install --requires=dep/0.1 -s build_type=Debug")
        run_pkg("dep/0.1")

    # Do a source change in the editable!
    with c.chdir("dep"):
        dep = c.load("src/dep.cpp")
        dep = dep.replace("dep/0.1", "SUPERDEP/0.1")
        c.save({"src/dep.cpp": dep})
        build_dep()

    with c.chdir("pkg"):
        run_pkg("SUPERDEP/0.1")


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.parametrize("generator", [None, "MinGW Makefiles"])
@pytest.mark.tool("mingw64")
def test_editable_cmake_windows_exe(generator):
    editable_cmake_exe(generator)


@pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
@pytest.mark.parametrize("generator", [None, "Ninja", "Ninja Multi-Config"])
@pytest.mark.tool("cmake", "3.19")
def test_editable_cmake_linux_exe(generator):
    editable_cmake_exe(generator)


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Macos")
@pytest.mark.parametrize("generator", [None, "Ninja", "Xcode"])
@pytest.mark.tool("cmake", "3.23")
def test_editable_cmake_osx_exe(generator):
    editable_cmake_exe(generator)



import os

from conan.cli.formatters.report import format_diff_html, format_diff_txt, format_diff_json
from conan.api.conan_api import ConanAPI
from conan.cli.command import conan_command, conan_subcommand



@conan_command(group="Security")
def report(conan_api: ConanAPI, parser, *args):
    """
    Gets information about the recipe and its sources.
    """


@conan_subcommand(formatters={"text": format_diff_txt,
                              "json": format_diff_json,
                              "html": format_diff_html})
def report_diff(conan_api, parser, subparser, *args):
    """
    Get the difference between two recipes with their sources.
    It can be used to compare two different versions of the same recipe, or two different recipe revisions.

    Each old/new recipe can be specified by a path to a conanfile.py and a companion reference,
    or by a reference only.

    If only a reference is specified, it will be searched in the local cache,
    or downloaded from the specified remotes. If no revision is specified, the latest revision will be used.
    """

    ref_help = ("{type} reference, e.g. 'mylib/1.0'. "
                "If used on its own, it can contain a revision, which will be resolved to the latest one if not provided, "
                "but it will be ignored if a path is specified. "
                "If used with a path, it will be used to create the reference for the recipe to be compared.")

    subparser.add_argument("-op", "--old-path", help="Path to the old recipe if comparing a local recipe is desired")
    subparser.add_argument("-or", "--old-reference", help=ref_help.format(type="Old"), required=True)

    subparser.add_argument("-np", "--new-path", help="Path to the new recipe if comparing a local recipe is desired")
    subparser.add_argument("-nr", "--new-reference", help=ref_help.format(type="New"), required=True)

    subparser.add_argument("-r", "--remote", action="append", default=None,
                       help='Look in the specified remote or remotes server')

    args = parser.parse_args(*args)

    cwd = os.getcwd()
    enabled_remotes = conan_api.remotes.list(args.remote or "*")

    result = conan_api.report.diff(args.old_reference, args.new_reference, enabled_remotes,
                                   old_path=args.old_path, new_path=args.new_path, cwd=cwd)
    return {
        "conan_api": conan_api,
        "diff": result["diff"],
        "old_export_ref": result["old_export_ref"],
        "new_export_ref": result["new_export_ref"],
        "old_cache_path": result["old_cache_path"],
        "new_cache_path": result["new_cache_path"],
        "src_prefix": result["src_prefix"],
        "dst_prefix": result["dst_prefix"],
    }



import pytest
import os
from unittest.mock import patch

from conan.tools.build import save_toolchain_args
from conan.tools.gnu import Autotools
from conan.test.utils.mocks import ConanFileMock, MockSettings
from conan.test.utils.test_files import temp_folder


@patch('conan.tools.gnu.autotools.chdir')
def test_source_folder_works(chdir_mock):
    folder = temp_folder()
    os.chdir(folder)
    save_toolchain_args({
        "configure_args": "-foo bar",
        "make_args": "",
        "autoreconf_args": "-bar foo"}
    )
    conanfile = ConanFileMock()
    sources = "/path/to/sources"
    conanfile.folders.set_base_source(sources)
    autotools = Autotools(conanfile)
    autotools.configure(build_script_folder="subfolder")
    assert conanfile.command.replace("\\", "/") == '"/path/to/sources/subfolder/configure" -foo bar '

    autotools = Autotools(conanfile)
    autotools.configure()
    assert conanfile.command.replace("\\", "/") == '"/path/to/sources/configure" -foo bar '

    autotools.autoreconf(build_script_folder="subfolder")
    chdir_mock.assert_called_with(autotools,
                                  os.path.normpath(os.path.join("/path/to/sources", "subfolder")))

    autotools.autoreconf()
    chdir_mock.assert_called_with(autotools, os.path.normpath("/path/to/sources"))
    assert conanfile.command == 'autoreconf -bar foo'


@pytest.mark.parametrize("install_strip", [False, True])
def test_install_strip(install_strip):
    """
    When the configuration `tools.build:install_strip` is set to True,
    the Autotools install command should invoke the `install-strip` target.
    """
    folder = temp_folder()
    os.chdir(folder)
    save_toolchain_args({})
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "7",
                             "os": "Linux",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.conf.define("tools.build:install_strip", install_strip)
    conanfile.folders.generators = "."

    autotools = Autotools(conanfile)
    autotools.install()

    assert ('install-strip' in str(conanfile.command)) == install_strip



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_editable_layout_paths():
    # https://github.com/conan-io/conan/issues/12521
    # https://github.com/conan-io/conan/issues/12839
    c = TestClient()
    dep = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                self.cpp.source.includedirs = ["include"]
            """)
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_settings("build_type", "arch")
                                                          .with_requires("dep/0.1")
                                                          .with_generator("CMakeDeps")
                                                          .with_generator("PkgConfigDeps")
                                                          .with_generator("XcodeDeps")})
    c.run("editable add dep")
    c.run("install pkg -s arch=x86_64")
    # It doesn't crash anymore

    assert "dep/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709 - Editable" in c.out
    data = c.load(f"pkg/dep-release-x86_64-data.cmake")

    assert 'set(dep_INCLUDE_DIRS_RELEASE "${dep_PACKAGE_FOLDER_RELEASE}/include")' in data
    pc = c.load("pkg/dep.pc")
    assert "includedir=${prefix}/include" in pc
    xcode = c.load("pkg/conan_dep_dep_release_x86_64.xcconfig")
    dep_path = os.path.join(c.current_folder, "dep")
    assert f"PACKAGE_ROOT_dep[config=Release][arch=x86_64][sdk=*] = {dep_path}" in xcode


def test_layout_paths_normalized():
    # make sure the paths doesn't end with trailing ".", and they are identical to the cwd
    c = TestClient()
    dep = textwrap.dedent("""
        import os
        from conan import ConanFile

        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                self.folders.source = "."
                self.folders.build = "."
                self.folders.generators = "."

            def build(self):
                assert os.getcwd() == self.source_folder
                assert os.getcwd() == self.build_folder
                assert os.getcwd() == self.generators_folder

            """)
    c.save({"conanfile.py": dep})
    c.run("build .")
    # It doesn't assert in the build()



import platform
import re
import textwrap
import os
import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.tools.apple.apple import XCRun
from conan.test.utils.mocks import ConanFileMock, MockSettings
from conan.internal.util.runners import conan_run


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Xcode")
def test_xcrun():
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.apple import XCRun

        class HelloConan(ConanFile):
            name = "hello"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            def build(self):
                sdk_path = XCRun(self).sdk_path
                self.output.info(sdk_path)
        """)
    client = TestClient(path_with_spaces=False)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create .")
    assert "MacOSX" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires OSX and xcrun tool")
def test_xcrun_sdks():
    def _common_asserts(xcrun_):
        assert xcrun_.cc.endswith('clang')
        assert xcrun_.cxx.endswith('clang++')
        assert xcrun_.ar.endswith('ar')
        assert xcrun_.ranlib.endswith('ranlib')
        assert xcrun_.strip.endswith('strip')
        assert xcrun_.find('lipo').endswith('lipo')
        assert os.path.isdir(xcrun_.sdk_path)

    conanfile = ConanFileMock( runner=conan_run)
    conanfile.settings = MockSettings(
        {"os": "Macos",
         "arch": "x86"})
    xcrun = XCRun(conanfile)
    _common_asserts(xcrun)

    conanfile.settings = MockSettings(
        {"os": "iOS",
         "arch": "x86"})
    xcrun = XCRun(conanfile, sdk='macosx')
    _common_asserts(xcrun)
    # Simulator
    assert "iPhoneOS" not in xcrun.sdk_path

    conanfile.settings = MockSettings(
        {"os": "iOS",
         "os.sdk": "iphoneos",
         "arch": "armv7"})
    xcrun = XCRun(conanfile)
    _common_asserts(xcrun)
    assert "iPhoneOS" in xcrun.sdk_path

    conanfile.settings = MockSettings(
        {"os": "watchOS",
         "os.sdk": "watchos",
         "arch": "armv7"})
    xcrun = XCRun(conanfile)
    _common_asserts(xcrun)
    assert "WatchOS" in xcrun.sdk_path

    # Default one
    conanfile.settings = MockSettings({})
    xcrun = XCRun(conanfile)
    _common_asserts(xcrun)


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Xcode")
def test_xcrun_in_tool_requires():
    # https://github.com/conan-io/conan/issues/12260
    client = TestClient()
    tool = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.apple import XCRun
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"

            def package_info(self):
                xcrun = XCRun(self{})
                self.output.info("sdk: %s" % xcrun.sdk_path)
        """)
    client.save({"br.py": tool.format(", use_settings_target=True")})
    client.run("export br.py --name=br --version=0.1")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            tool_requires = "br/0.1"
        """)

    profile_ios = textwrap.dedent("""
        include(default)
        [settings]
        os=iOS
        os.version=15.5
        os.sdk=iphoneos
        os.sdk_version=15.5
        arch=armv8
    """)

    client.save({"conanfile.py": conanfile, "profile_ios": profile_ios})
    client.run("create . --name=pkg --version=1.0 -pr:h=./profile_ios -pr:b=default --build='*'")
    assert re.search("sdk:.*iPhoneOS", str(client.out))
    assert not re.search("sdk:.*MacOSX", str(client.out))

    client.save({"br.py": tool.format("")})
    client.run("export br.py --name=br --version=0.1")
    client.run("create . --name=pkg --version=1.0 -pr:h=./profile_ios -pr:b=default --build='*'")
    assert not re.search("sdk:.*iPhoneOS", str(client.out))
    assert re.search("sdk:.*MacOSX", str(client.out))


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Xcode")
def test_xcrun_in_required_by_tool_requires():
    """
    ConanCenter case, most typical, openssl builds with autotools so needs the sysroot
    and is a require by cmake so in the build context it needs the settings_build, not
    the settings_target, that's why the use_settings_target default is False
    """
    client = TestClient()

    openssl = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.apple import XCRun
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def build(self):
                xcrun = XCRun(self)
                self.output.info("sdk for building openssl: %s" % xcrun.sdk_path)
        """)

    consumer = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            tool_requires = "cmake/1.0"
        """)

    profile_ios = textwrap.dedent("""
        include(default)
        [settings]
        os=iOS
        os.version=15.5
        os.sdk=iphoneos
        os.sdk_version=15.5
        arch=armv8
    """)

    client.save({"cmake.py": GenConanfile("cmake", "1.0").with_requires("openssl/1.0"),
                 "openssl.py": openssl,
                 "consumer.py": consumer,
                 "profile_ios": profile_ios})

    client.run("export openssl.py --name=openssl --version=1.0")
    client.run("export cmake.py")
    client.run("create consumer.py --name=consumer --version=1.0 -pr:h=./profile_ios -pr:b=default --build='*'")

    assert re.search("sdk for building openssl:.*MacOSX", str(client.out))



# coding=utf-8

import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.artifactory_ready
def test_conaninfo_special_chars():

    t = TestClient(default_server_user=True)
    conanfile = textwrap.dedent("""
    # coding=utf-8
    from conan import ConanFile

    class Recipe(ConanFile):
        name = "weird_info"
        version = "1.0"
        options = {"ññ¨¨&是": ['是"是', '][{}"是是是']}
        default_options = {"ññ¨¨&是": '][{}"是是是'}
    """)

    t.save({"conanfile.py": conanfile})
    t.run("create . ")
    t.run('list weird_info/1.0:*')
    assert 'ññ¨¨&是: ][{}"是是是' in t.out

    t.run("upload * -c -r default")
    # TODO: I have struggled with this, it was not accepting "latest", revision needs explicit one
    t.run('list weird_info/1.0#8c9e59246220eef8ca3bd4ac4f39ceb3:* -r default')
    assert 'ññ¨¨&是: ][{}"是是是' in t.out



import os

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, NO_SETTINGS_PACKAGE_ID
from conan.test.utils.env import environment_update


@pytest.mark.parametrize("requires", ["requires", "tool_requires"])
def test_lock_packages(requires):
    """
    Check that package revisions can be locked too
    NOTE: They are still not used! only to check that it is possible to store them
          And that the lockfile is still usable
    """
    client = TestClient(light=True)
    client.save({"pkg/conanfile.py": GenConanfile().with_package_file("file.txt", env_var="MYVAR"),
                 "consumer/conanfile.txt": f"[{requires}]\npkg/[>0.0]"})
    with environment_update({"MYVAR": "MYVALUE"}):
        client.run("create pkg --name=pkg --version=0.1")
    prev = client.created_package_revision("pkg/0.1")

    client.run("lock create consumer/conanfile.txt --lockfile-packages")
    assert "ERROR: The --lockfile-packages arg is private and shouldn't be used" in client.out
    assert "pkg/0.1#" in client.out
    lock = client.load("consumer/conan.lock")
    assert NO_SETTINGS_PACKAGE_ID in lock

    with environment_update({"MYVAR": "MYVALUE2"}):
        client.run("create pkg --name=pkg --version=0.1")
    prev2 = client.created_package_revision("pkg/0.1")
    assert prev2 != prev

    client.run("install consumer/conanfile.txt")
    assert prev in client.out
    assert prev2 not in client.out

    os.remove(os.path.join(client.current_folder, "consumer/conan.lock"))
    client.run("install consumer/conanfile.txt")
    assert prev2 in client.out
    assert prev not in client.out
